

#include "my_defs.h"
#include "serial.h"
#include "lcd.h"
#include "rc5.h"
#include "fat.h"
#include "ide.h"
#include "tools.h"
#include "i2c.h"
#include "mas.h"
#include "progmem.h"
#include "memory.h"

#if DEBUG == TRUE

void IdeTest (void);
void LcdTest(void);
void RC5Test(void);
void I2CTest(void);
void MASTest(void);											 
void PortTest(void);
void HDStrukturTest(void);
void MemoryTest(void);
unsigned char dir_debug( unsigned long);
void read_lba_debug( unsigned long, unsigned char, unsigned char*, unsigned char);
void init_fat_debug( void);
void WeiterMitTaste(void);
extern unsigned char* name(void);
extern unsigned long * get_song(void);

extern unsigned int SaveOffset;
extern unsigned long FAT_START;
extern unsigned long DIR_START;		
extern unsigned int Fat32Length;		//Anzahl in Sektoren
extern unsigned long DataStart;
extern unsigned char DirCount;
extern unsigned char Deep;

unsigned char  ClusterSize;


int main( void )
{
    outp((0<<RXCIE) |(0<<TXCIE)|(1<<RXEN) |(1<<TXEN),UCR);    	// enable TX and RX
    outp(UART_BAUD_SELECT, UBRR); 								// set baudrate


	//diverse Interrupteinstellungen, ext. RAM
#if MINIMEGAHMPEG == FALSE	// Hmpeg
    outp( (1<<SRE) 						// external SRAM enable
    	| (1<<ISC00) | (1<<ISC01) 		// rising edge of INT0 (DEM) generates an irq
    	| (1<< ISC10) | (1<<ISC11)		// rising edge of INT1 generates an irg
    	, MCUCR);
	timer0_source(CK256);	// Timer0 Clock = F_CPU / 256
#else	// MiniMegaHmpeg
    outp(1<<SRE, MCUCR);					// external SRAM enable
    outp((1<<ISC41) | (1<<ISC40) , EICR);	// rising edge of INT4 (DEM) generates an irq 
	timer2_source(CK256);	// Timer2 Clock = F_CPU / 256
#endif

	lcd_init();
	i2c_init();
	mas_init();
	ide_init();

#if MINIMEGAHMPEG == FALSE	// Hmpeg
	lcd_nprint_(PSTR ("HD DEBUG 38400 8N1!"),0xff);
#else	// MiniMegaHmpeg
	lcd_nprint_(PSTR ("HD DEBUG 9600 8N1!"),0xff);
#endif

	while (1)
	{	
		sendstring_(PSTR ("[2JHD DEBUG V0.98m\n\r"));
		sendstring_(PSTR("Test von: \n\r"));
		sendstring_(PSTR("IDE\t\t\t1\n\r"));
		sendstring_(PSTR("LCD\t\t\t2\n\r"));
		sendstring_(PSTR("RC5\t\t\t3\n\r"));
		sendstring_(PSTR("I2C\t\t\t4\n\r"));
		sendstring_(PSTR("MAS\t\t\t5\n\r"));
		sendstring_(PSTR("Ports\t\t\t6\n\r"));
		sendstring_(PSTR("Verzeichnisstruktur\t7\n\r"));			
		sendstring_(PSTR("Speichertest\t\t8\n\r"));			
		while (bit_is_clear(USR,RXC));		//ein Byte von der RS232 empfangen worden?
		sendstring_(PSTR ("[2JTest "));
		switch(inp(UDR))
		{
			case '1':
				IdeTest();
			break;
			case '2':
				LcdTest();
			break;
			case '3':
				RC5Test();
			break;
			case '4':
				I2CTest();
			break;
			case '5':
				MASTest();
			break;
			case '6':
				PortTest();
			break;
			case '7':
				HDStrukturTest();
			break;
			case '8':
				MemoryTest();
			break;
		}				
	}
}

void IdeTest (void)
{
	sendstring_(PSTR ("IDE\n\r"));	
	while (1)
	{	
		sendstring_(PSTR("\n\rRESET\t\t\t1\n\r"));
		sendstring_(PSTR("INIT FAT\t\t2\n\r"));
		sendstring_(PSTR("READ Register\t\t3\n\r"));
		sendstring_(PSTR("READ LBA 0\t\t4\n\r"));
		sendstring_(PSTR("LESE DATL/H\t\t5\n\r"));		
		sendstring_(PSTR("MASTER\t\t\t6\n\r"));
		sendstring_(PSTR("SLAVE\t\t\t7\n\r"));		
		sendstring_(PSTR("Ende\t\t\tE\n\r"));	

  		while (bit_is_clear(USR,RXC));		//ein Byte von der RS232 empfangen worden?
  		switch(inp(UDR))
  		{
  			case '1':
				*CB_DH =  DEVICE;
				*CB_DC = CB_DC_HD15 | CB_DC_NIEN ;
				sendstring_(PSTR ("\n\rRESET!"));
				ide_init();
  			break;
  			case '2':
				init_fat_debug();
				sendstring_(PSTR ("\n\rFAT Start: "));	print_long(FAT_START);
				sendstring_(PSTR ("\n\rDIR Start: "));	print_long(DIR_START);	
				sendstring_(PSTR ("\n\rFAT Lnge: "));	print_long(Fat32Length);	
  			break;
  			case '3':
				dump_ide_reg();
  			break;
  			case '4':
				read_lba_debug( 0, 1, (unsigned char*) SECTORBUFFER_ADR, 0);
				show_mem( (unsigned char*) SECTORBUFFER_ADR, (unsigned char*) (0x8000));
  			break;
  			case '5':
				while (bit_is_clear(USR,RXC))
				{	
					sendstring( "DL:");
					sendstring( byte_to_hex(*CB_DATA) );
					sendstring( " DH:");
					sendstring( byte_to_hex(*DATA_H) );
					sendstring_(PSTR ("[11D"));	
				}
  			break;
  			case '6':
				DEVICE = CB_DH_DEV0;
				*CB_DH =  DEVICE;
				*CB_DC = CB_DC_HD15 | CB_DC_NIEN ;				
				sendstring_(PSTR ("aktuelle Platte: MASTER"));	
  			break;
  			case '7':
				DEVICE = CB_DH_DEV1;
				*CB_DH =  DEVICE;
				*CB_DC = CB_DC_HD15 | CB_DC_NIEN ;				
				sendstring_(PSTR ("aktuelle Platte: SLAVE"));	
  			break;

  			case 'e':
  			case 'E':
			return;
  		}
	} 
}

void LcdTest(void)
{
	while(1)
	{	unsigned char n;
		sendstring_(PSTR ("LCD"));
		sendstring_(PSTR("\n\rINIT\t\t\t1\n\r"));
		sendstring_(PSTR("Daten schreiben\t\t2\n\r"));
		sendstring_(PSTR("Daten lesen\t\t3\n\r"));
		sendstring_(PSTR("Ende\t\t\tE\n\r"));	
  		while (bit_is_clear(USR,RXC));		//ein Byte von der RS232 empfangen worden?
  		switch(inp(UDR))
  		{
  			case '1':
				lcd_init();
  			break;
  			case '2':
				sendstring_(PSTR("schreibe... Ende mit bel. Taste\n\r"));
				while (bit_is_clear(USR,RXC))
				{	for (n=0; n<255; n++)
					{	delay_ms(10);
						GO_WS
						*LCD_DATA = n;
						END_WS
					}
				}
				inp(UDR);
  			break;
  			case '3':
				while (bit_is_clear(USR,RXC))
				{	delay_ms(10);
					GO_WS
					sendstring(byte_to_hex( *LCD_DATA));
					END_WS
					sendstring_(PSTR("  "));					
				}
				inp(UDR);
  			break;
  			case 'E':
  			case 'e':			
				return;
  		}
	} 
}

void RC5Test(void)
{
unsigned int code;
	sendstring_(PSTR ("RC5\n\r"));	
	while (1)
	{	
		sendstring_(PSTR("\n\rCode lesen\t\t1\n\r"));
		sendstring_(PSTR("Pegel anzeigen\t\t2\n\r"));
		sendstring_(PSTR("Ende\t\t\tE\n\r"));	
  		while (bit_is_clear(USR,RXC));		//ein Byte von der RS232 empfangen worden?
  		switch(inp(UDR))
  		{
  			case '1':
				sendstring_(PSTR("warte...\n\r"));
				while (bit_is_clear(USR,RXC))
				{	if (REMOTE_CLEAR) 
					{	code = get_rc5();
						sendstring_(PSTR("\n\rCode gelesen: "));
						print_long(code);
					}
				}
  			break;
  			case '2':
				while (bit_is_clear(USR,RXC))
				{	if (REMOTE_CLEAR)
					{	sendstring_(PSTR("0 "));
						while (REMOTE_CLEAR);
					}
					else 
					{	sendstring_(PSTR("1 "));
						while (REMOTE_SET);
					}
				}
  			break;
  			case 'e':						
  			case 'E': return;			
  		}
	} 
}

void I2CTest(void)
{
	sendstring_(PSTR ("I2C, Ende mit bel. Taste\n\r"));	
	while (bit_is_clear(USR,RXC))
	{	
		sendstring_(PSTR ("I2CC:"));	
		putchar (0x30 + I2CC_SET);
		sendstring_(PSTR (" I2CD:"));	
		putchar (0x30 + I2CD_SET);
		sendstring_(PSTR ("[15D"));	
	}
}

void MASTest(void)
{
	sendstring_(PSTR ("der Signale am MAS, Ende mit bel. Taste\n\r"));	
	while (bit_is_clear(USR,RXC))
	{	
		sendstring_(PSTR ("Demand:"));	
		putchar (0x30 + DEMAND_SET);
		sendstring_(PSTR (" Reset:"));	
		putchar (0x30 + RESET_SET);
		sendstring_(PSTR (" SID:"));	
		putchar (0x30 + SID_SET);
		sendstring_(PSTR (" SIC:"));	
		putchar (0x30 + SIC_SET);
		sendstring_(PSTR ("[32D"));	
	}
}

void PortTest(void)
{
	sendstring_(PSTR ("der PORTs, Ende mit bel. Taste\n\r"));	
	while (bit_is_clear(USR,RXC))
	{	
#if MINIMEGAHMPEG == FALSE	// Hmpeg
		sendstring_(PSTR ("B2:"));	
		putchar (0x30 + bit_is_set(PINB, PINB2));
		sendstring_(PSTR (" B3:"));	
		putchar (0x30 + bit_is_set(PINB, PINB3));
		sendstring_(PSTR (" D3:"));	
		putchar (0x30 + bit_is_set(PIND, PIND3));
		sendstring_(PSTR ("[17D"));	
#else	// MiniMegaHmpeg
		sendstring_(PSTR ("B4:"));	
		putchar (0x30 + bit_is_set(PINB, PINB4));
		sendstring_(PSTR (" B5:"));	
		putchar (0x30 + bit_is_set(PINB, PINB5));
		sendstring_(PSTR (" B6:"));	
		putchar (0x30 + bit_is_set(PINB, PINB6));
		sendstring_(PSTR (" B7:"));	
		putchar (0x30 + bit_is_set(PINB, PINB7));

		sendstring_(PSTR ("\r\nD0:"));	
		putchar (0x30 + bit_is_set(PIND, PIND0));
		sendstring_(PSTR (" D1:"));	
		putchar (0x30 + bit_is_set(PIND, PIND1));
		sendstring_(PSTR (" D2:"));	
		putchar (0x30 + bit_is_set(PIND, PIND2));
		sendstring_(PSTR (" D3:"));	
		putchar (0x30 + bit_is_set(PIND, PIND3));
		sendstring_(PSTR (" D4:"));	
		putchar (0x30 + bit_is_set(PIND, PIND4));

		sendstring_(PSTR ("\r\nE2:"));	
		putchar (0x30 + bit_is_set(PINE, PINE2));
		sendstring_(PSTR (" E3:"));	
		putchar (0x30 + bit_is_set(PINE, PINE3));
		sendstring_(PSTR (" E5:"));	
		putchar (0x30 + bit_is_set(PINE, PINE5));
		sendstring_(PSTR (" E6:"));	
		putchar (0x30 + bit_is_set(PINE, PINE6));
		sendstring_(PSTR (" E7:"));	
		putchar (0x30 + bit_is_set(PINE, PINE7));

		sendstring_(PSTR ("\r\nF0:"));	
		putchar (0x30 + bit_is_set(PINF, PINF0));
		sendstring_(PSTR (" F1:"));	
		putchar (0x30 + bit_is_set(PINF, PINF1));
		sendstring_(PSTR (" F2:"));	
		putchar (0x30 + bit_is_set(PINF, PINF2));
		sendstring_(PSTR (" F3:"));	
		putchar (0x30 + bit_is_set(PINF, PINF3));
		sendstring_(PSTR (" F4:"));	
		putchar (0x30 + bit_is_set(PINF, PINF4));
		sendstring_(PSTR (" F5:"));	
		putchar (0x30 + bit_is_set(PINF, PINF5));
		sendstring_(PSTR (" F6:"));	
		putchar (0x30 + bit_is_set(PINF, PINF6));
		sendstring_(PSTR (" F7:"));	
		putchar (0x30 + bit_is_set(PINF, PINF7));

		sendstring_(PSTR ("[2;1H"));	
#endif
	}	
}

void MemoryTest(void)
{
	unsigned char* RAM = (unsigned char*)ARG_BUFFER_ADR;
	unsigned int n;
	unsigned char i;

	sendstring_(PSTR ("Memory: "));	
	i=0;
	do 
	{
		memset((unsigned int *)ARG_BUFFER_ADR, i, 0x8000-ARG_BUFFER_ADR);
		for (n=0; n < 0x8000-ARG_BUFFER_ADR; n++)
		{	if (RAM[n] != i) 
			{	sendstring_(PSTR("failed"));
				while(2);
			}
		}
		if ((i++ & 0x07)==0)	putchar ('.');
	}	while (i);	
	sendstring_(PSTR("OK!"));
	while (bit_is_clear(USR,RXC));	
}

void HDStrukturTest(void)
{
#define MaxPathDeep 8
static unsigned char LastEntry[MaxPathDeep];
static unsigned long LastDir[MaxPathDeep];
static unsigned char LastDirEnde[MaxPathDeep];
extern unsigned char Entry;
unsigned long ActualPath = 0;
unsigned char* DirName = (unsigned char *)DIRNAME_ADR;

	*(unsigned int *)DIRNAME_ADR = 0;
	sendstring_(PSTR ("der Verzeichnisstruktur\n\rInit Fat: "));	
	DEVICE = CB_DH_DEV0;
	ide_init();
	init_fat_debug();	
	Entry = -1;
	sendstring_(PSTR ("\r\nLese Root:\n\r"));
	if (dir_debug(ActualPath) != 2)
	{
		while (1)
		{
			if (++Entry < DirCount)					//nchsten Eintrag bearbeiten
			{	if (*name() == DIR_MARK)			//es handelt sich um ein Dir!
				{	LastDir[Deep] = ActualPath;		//merken fr die Rckkehr
					LastDirEnde[Deep] = strlen(DirName);	//merken fr die Rckkehr
					strcat(DirName,name());
					sendstring_(PSTR ("\n\rDIR:"));
					sendstring(DirName);
					new_line();
					LastEntry[Deep++] = Entry;		//hier haben wir aufgehrt
					if (Deep == MaxPathDeep)
					{	sendstring_(PSTR ("\n\rZu viele Unterverzeichnisse."));
						break;
					}
					ActualPath = *get_song();		//hier jetzt weitermachen Dir
					if (dir_debug(ActualPath) == 2) break;			
					sendstring_(PSTR ("Belegt im Dirbuffer: "));	print_long(SaveOffset);
					sendstring_(PSTR (" Bytes\n\r"));
					if (SaveOffset > 6160)
					{ 	sendstring_(PSTR ("\n\rZu viel!"));	
					 	break;
					}
					Entry = -1;						//und hier gehts weiter	File
				}
				else 
				{	if (Deep == 0) DirName[1] =0;
					sendstring(name());
					new_line();
				}
			}
			else 
			{	if (Deep !=0)
				{
					ActualPath = LastDir[--Deep];	//das war der letzte Pfad
					Entry = LastEntry[Deep];		//hier hatten wir aufgehrt
					DirName[LastDirEnde[Deep]] =0; 
					dir_debug(ActualPath);					
				}
				else 
				{
					if (SlavePresent && (DEVICE == CB_DH_DEV0))
					{	inp(UDR);
						sendstring_(PSTR ("\n\r Master Fertig - weiter mit Slave\n\r"));	
						while (bit_is_clear(USR,RXC));
						DEVICE = CB_DH_DEV1;			//nchste Platte
						init_fat_debug(); 				
						ActualPath = 0;					//das ist der erste Pfad
						dir_debug(ActualPath);		  	//Root reinholen		
						Entry = -1;						//Wird weiter oben incrementiert
						*DirName =0; 
					}
					else 
						break;
				}
			}
		}		
	}
	WeiterMitTaste();
}

unsigned char inline read_fat_sector_debug( unsigned long d_sector)
{
	unsigned char dummy;
	struct msdos_dir_entry *entry;
	struct msdos_dir_slot *ms_slot;
	unsigned char TrackNummer;
	unsigned long * l_ptr;
	unsigned char * w_buffer 	= (unsigned char *) W_BUFFER_ADR;	
	unsigned int *NameBuffer 	= (unsigned int *)	NAMEBUFFER_ADR;
	unsigned char *DirBuffer	= (unsigned char *) DIRBUFFER_ADR;
		
	read_lba_debug(d_sector, 1, (unsigned char*) SECTORBUFFER_ADR, 1);	//read directory sector
	entry = (struct msdos_dir_entry *) SECTORBUFFER_ADR;

	dummy =-1;
	while(++dummy < 16)		//max 16 entries
	{
		switch( entry[dummy].name[0])
		{	case 0:		return 1;				//last entry ?
			case 0x2E:							//Dir: . or .. we don't need
			case 0xE5:							//File deleted
				*w_buffer = 0x00;
				continue;
		}

		if( (entry[dummy].attr & ATTR_SLOT) == ATTR_SLOT)	//Win95 DirSlot ?
		{
			ms_slot =(struct msdos_dir_slot *)&entry[dummy];
			strcpy((unsigned char *)W1_BUFFER_ADR, w_buffer);			//save old buffer				
			ws_strncpy( w_buffer,  (unsigned int*)&ms_slot->name0_4[0], 5);		//copy ldn name 0-4
			ws_strncpy( &w_buffer[5],  (unsigned int*)&ms_slot->name5_10[0], 6);//copy ldn name 5-10
			ws_strncpy( &w_buffer[11], (unsigned int*)&ms_slot->name11_12[0], 2);//copy ldn name 11-12
			strcpy(&w_buffer[13],(unsigned char *)W1_BUFFER_ADR);
			continue;
		}
		
		if( (entry[dummy].attr & (ATTR_HIDDEN | ATTR_VOLUME)) != 0)
		{
			*w_buffer = 0x00;
			continue;
		}

		if( *w_buffer == 0x00)			//long filename check
			strncpy(w_buffer,entry[dummy].name,11);

		if (strcmp(w_buffer,"playlist.m3u") == 0)
		{	continue;
		}

		l_ptr = (unsigned long *) &DirBuffer[SaveOffset];
		l_ptr[0] = ((unsigned long) entry[dummy].starthi <<16) | entry[dummy].start; //save Cluster
		l_ptr[1] = entry[dummy].size;			//save size
		

		if ((w_buffer[0] > 0x39) || (w_buffer[0] < 0x30) ||
			(w_buffer[1] > 0x39) || (w_buffer[1] < 0x30)) 
		{	
			sendstring_(PSTR("\n\rFehlende Nr. im Dateinamen: "));		
			sendstring(w_buffer);		
			WeiterMitTaste();
		}

		TrackNummer = (w_buffer[0] - 0x30) *10 + w_buffer[1] - 0x31;		

		if (NameBuffer[TrackNummer] == 0xFFFF)
		{	
			NameBuffer[TrackNummer] = SaveOffset;
		}
		else 
		{	
			sendstring_(PSTR("\n\rNr. doppelt bei: "));		
			sendstring(w_buffer);		
			WeiterMitTaste();
		}
		
		SaveOffset += (2* sizeof(unsigned long));
		
		if((entry[dummy].attr & ATTR_DIR) == ATTR_DIR)
		{  	
			DirBuffer[SaveOffset++] = DIR_MARK;			
		}
		else if (strcmp(&w_buffer[strlen(w_buffer)-3],"mp3") != 0)
		{	
			sendstring_(PSTR("\n\rMP3?: "));		
			sendstring(w_buffer);
			WeiterMitTaste();
		}
		
		strcpy( &DirBuffer[SaveOffset], w_buffer);				//save filename
		SaveOffset += strlen(w_buffer) +1;						//+1 for 0
		*w_buffer = 0x00;
		DirCount++;
	}
	return 0;
}

unsigned char dir_debug( unsigned long cluster)
{
	unsigned long sec_to_read;
	unsigned long first_sector;
	unsigned int *NameBuffer 	= (unsigned int *) 	NAMEBUFFER_ADR;
	unsigned int n = 0;	
	DirCount = 0;
	SaveOffset = 0;
	*(unsigned char *)W_BUFFER_ADR =0;

	for (n =0; n < 256; n++)
	{
		NameBuffer [n] =0xFFFF;
	} 

	if (cluster == 0)
	{
		first_sector = DIR_START;		//read ROOT dir
		sec_to_read = Fat32Length;
	}
	else
	{
		first_sector = cluster_to_sector( cluster);		//read subdir
		sec_to_read = ClusterSize;
	}

	while(1)
	{	unsigned long sector;
		unsigned char c;
		for(sector = 0; sector < sec_to_read; sector++)
		{	
			c = read_fat_sector_debug( first_sector + sector);
			if  (c)
			{
				for (n=0; n < DirCount; n++)
					if (NameBuffer[n] == 0xFFFF)
					{	sendstring_(PSTR("\n\rNummer im Verzeichnis nicht vorhanden: "));
						print_byte_dec(n+1);
						WeiterMitTaste();
					}
			return c;
			}
		}		
		cluster = get_next_cluster(cluster);
		first_sector = cluster_to_sector(cluster);
	}
}

void read_lba_debug( unsigned long lba, unsigned char sec_count, unsigned char *in_buffer, unsigned char silent)
{
unsigned int dummy=0;
unsigned int n =0;
	*CB_DH = DEVICE;
	*CB_DC = CB_DC_HD15 | CB_DC_NIEN ;	
	if (!silent)sendstring_(PSTR("Warte auf Busy. "));	
	while( (*CB_STAT & CB_STAT_BSY) == CB_STAT_BSY) 
	{	delay_ms (10);
		if (++n == 4000)
		{	sendstring_(PSTR("Timeout!\n\r"));	
			return;
		}
	}
	if (!silent) sendstring_(PSTR("Ok.\n\r"));	
	*CB_DH = ((lba>>24) & 0x0f) | DEVICE;

	n=0;
	//wait for DRDY == 1 && SEEK_COMPLETE==1
	if (!silent) sendstring_(PSTR("Warte auf Seek complete. "));	
	while((*CB_STAT & ( CB_STAT_BSY | CB_STAT_RDY | CB_STAT_SKC ))	!= (CB_STAT_RDY | CB_STAT_SKC))
	{	delay_ms (10);
		if (++n == 4000)
		{	sendstring_(PSTR("Timeout!\n\r"));	
			return;
		}
	}
	if (!silent)sendstring_(PSTR("Ok.\n\r"));	
	*CB_FR = 0;
	*CB_SC = sec_count;
	*CB_SN = lba;	
	*CB_CL = lba>>8;
	*CB_CH = lba>>16;
	*CB_CMD = 0x20;
		
  	if (!silent) sendstring_(PSTR("Warte auf DRQ oder ERR!\n\r"));
	while(sec_count)
	{
		if( *CB_STAT & ( CB_STAT_DF | CB_STAT_ERR ))		
		{
			sendstring_(PSTR("HD error!\n\r"));
			break;
		}
		while (((*CB_STAT & (CB_STAT_DRQ | CB_STAT_BSY )) == CB_STAT_DRQ) && (sec_count))
		{
			in_buffer[dummy++] = *CB_DATA;
			in_buffer[dummy++] = *DATA_H;
			if ((dummy & 0x1ff) ==0) sec_count--;	

			if (*CB_STAT & CB_STAT_CORR)
			{	sendstring_(PSTR("Data corrected!\n\r"));
			}
		}
	}
}

//***********************************************************************

void init_fat_debug( void)
{
unsigned long first_sec;
	memset((unsigned char *)SECTORBUFFER_ADR, 0x00, 512);
	read_lba_debug(0, 1, (unsigned char*) SECTORBUFFER_ADR,0);

	if (*(unsigned int*) (SECTORBUFFER_ADR + MAGIC_OFFSET) != 0xAA55)
	{	sendstring_(PSTR ("Bootsektorkennung 55AA nicht gefunden.\n\r"));
		return;
	}
	first_sec = *(unsigned long*) (SECTORBUFFER_ADR + PAR_TABLE_OFFSET +8);

	if ((*(unsigned char*)(SECTORBUFFER_ADR + PAR_TABLE_OFFSET +4) != FAT32_LBA) 
		&& (*(unsigned char*)(SECTORBUFFER_ADR + PAR_TABLE_OFFSET +4) != FAT32))
	{	sendstring_(PSTR ("HD nicht FAT32 LBA - formatiert\n\r"));
		return;
	}
	read_lba_debug(first_sec, 1, (unsigned char*) SECTORBUFFER_ADR,0);	//read bootblock

	if (*(unsigned char*)(SECTORBUFFER_ADR + 13) != 16) 
	{	sendstring_(PSTR ("Falsche Clustergre\n\r"));		
		return;
	}
	ClusterSize	= 	 *(unsigned char*) (SECTORBUFFER_ADR +13);;
	FAT_START	     = first_sec + *(unsigned int*) (SECTORBUFFER_ADR + 14);
	Fat32Length      = *(unsigned int*) (SECTORBUFFER_ADR +36);
	DIR_START		 = FAT_START + Fat32Length * 2;
	DataStart 		 = DIR_START + *(unsigned char*)  (SECTORBUFFER_ADR +17);	

	*CB_DH = CB_DH_DEV1;										//Slave present?	
	SlavePresent = *CB_STAT;									//Slave not present: Stat = 0	
	
}
void WeiterMitTaste(void)
{
		inp(UDR);
		sendstring_(PSTR ("\n\rWeiter mit Taste\n\r"));	
		while (bit_is_clear(USR,RXC));
}


#endif
