#include <interrupt.h>
#include <io.h>					    
#include <eeprom.h>
#include <string-avr.h>

#include "my_defs.h"
#include "memory.h"
#include "serial.h"
#include "tools.h"

#include "ide.h"
#include "lcd.h"				 
#include "fat.h"
#include "mas.h"

unsigned char DirCount;					//number of entries in dir buffer
unsigned int  SaveOffset;
unsigned long DataStart;				//first sector of data

unsigned char DEVICE = CB_DH_DEV0;		//ide master = CB_DH_DEV0 or slave=CB_DH_DEV1 ?
unsigned char SlavePresent;				//true if slave present

//unsigned char ClusterSize;
unsigned long FAT_START;				//first fat sector 
unsigned long DIR_START;				//first dir sector
unsigned long Fat32Length;				//how many sectors
unsigned long first_cluster_in_cache;
unsigned long last_cluster;				

unsigned long PlaylistStartCluster;
unsigned long PlaylistLaenge;

extern volatile unsigned char TimeWindow;
extern volatile unsigned char Time;
extern void PumpData(void);
	
void get_fat(unsigned long cluster);
void read_lba(unsigned long ,unsigned char, unsigned char*);
unsigned char inline read_fat_sector(unsigned long);
void dir( unsigned long);
unsigned long cluster_to_sector( unsigned long);
unsigned long get_next_cluster( unsigned long);
void update_buf(unsigned char*);

/***********************************************************************************
- IDE CMD ausfueren; IMMER LESEN ! Nur LBA!
- Daten werden ausgelesen und nach "in_buffer" geschrieben
- IDE command "read", LBA only
- read number of sectors and write them into in_buffer
- lba: startpoint
- sec_count: number of sectors
- *in_buffer: destination
***********************************************************************************/
void read_lba( unsigned long lba, unsigned char sec_count, unsigned char *in_buffer)
{
unsigned int dummy=0;

	*CB_DH = DEVICE;						//Master or slave
	*CB_DC = CB_DC_HD15 | CB_DC_NIEN;		//no IDE interrupt!

	while( (*CB_STAT & CB_STAT_BSY) == CB_STAT_BSY);	//busy?

	//wait for DRDY == 1 && SEEK_COMPLETE==1
	while((*CB_STAT & ( CB_STAT_BSY | CB_STAT_RDY | CB_STAT_SKC ))	!= (CB_STAT_RDY | CB_STAT_SKC));

	*CB_DH = ((lba>>24) & 0x0f) | DEVICE;
	*CB_FR = 0;
	*CB_SC = sec_count;
	*CB_SN = lba;	
	*CB_CL = lba>>8;
	*CB_CH = lba>>16;
	*CB_CMD = 0x20;
		
	while(sec_count)
	{
		while (((*CB_STAT & (CB_STAT_DRQ | CB_STAT_BSY )) == CB_STAT_DRQ) && (sec_count))
		{
			in_buffer[dummy++] = *CB_DATA;
			in_buffer[dummy++] = *DATA_H;
			if (!(dummy & 0x1ff)) sec_count--;	
		}
	}
}

/***********************************************************************************
- liest die erste Partition auf der Platte 
- initialisiert globale Variablen fr den FAT32-Zugriff
- read only one partition
- initialize global variables
***********************************************************************************/
void init_fat( void)
{
unsigned long first_sec;

	read_lba(0, 1, (unsigned char*) SECTORBUFFER_ADR);			//read bootsector
	first_sec = *(unsigned long*) (SECTORBUFFER_ADR + PAR_TABLE_OFFSET +8);
	read_lba(first_sec, 1, (unsigned char*) SECTORBUFFER_ADR);	//read bootblock

//	ClusterSize	= SectorBuffer[13];
#define 	ClusterSize   16

	//Anfang berechnen
	FAT_START	     = first_sec + *(unsigned int*) (SECTORBUFFER_ADR + 14);
	Fat32Length      = *(unsigned int*) (SECTORBUFFER_ADR +36);
	DIR_START		 = FAT_START + Fat32Length * 2;
	DataStart 		 = DIR_START + *(unsigned char*) (SECTORBUFFER_ADR +17);	

	*CB_DH = CB_DH_DEV1;										//Slave present?	
	SlavePresent = *CB_STAT;									//Slave not present: Stat = 0	
}

/***********************************************************************************
- liest einen Sektor von einem Verzeichnis ein
- schreibt in den DirBuffer jede "normale" Datei und jedes "normale" Unterverzeichnis
- im DirBuffer landet 
	- unsigned long  StartCluster
	- unsigned long  Lnge
	- String mit dem Namen (Null-terminiert)
	- bei Verzeichnissen ein DirMark als erstes Zeichen im Namen
- return 0, wenn das Ende des Verzeichnisses nicht erreicht wurde, sonst return 1
***********************************************************************************/
unsigned char inline read_fat_sector( unsigned long d_sector)
{
	unsigned char TrackNummer;
	unsigned char Start;
	unsigned char *w_buffer = (unsigned char *) W_BUFFER_ADR;	
	unsigned int  *NameBuffer 	= (unsigned int *) 	NAMEBUFFER_ADR;
	unsigned char *DirBuffer	= (unsigned char *) DIRBUFFER_ADR;
	unsigned char *entry;

	read_lba(d_sector, 1, (unsigned char*) SECTORBUFFER_ADR);	//read directory sector

	//max 16 entries
	for (entry = (unsigned char*) SECTORBUFFER_ADR; entry < (unsigned char*)SECTORBUFFER_ADR+512 ;entry +=32)
	{
		switch( entry[0])
		{	case 0:		return 1;				//last entry ?
			case 0x2E:							//Dir: . or .. we don't need
			case 0xE5:							//File deleted
				*w_buffer = 0x00;
				continue;
		}

		if( (entry[11] & ATTR_SLOT) == ATTR_SLOT)						//Win95 DirSlot ?
		{
			strcpy((unsigned char *) W1_BUFFER_ADR, w_buffer);			//save old buffer				
			ws_strncpy( w_buffer,  (unsigned int*)&entry[1], 5);		//copy ldn name 0-4
			ws_strncpy( &w_buffer[5],  (unsigned int*)&entry[14], 6);	//copy ldn name 5-10
			ws_strncpy( &w_buffer[11], (unsigned int*)&entry[28], 2);	//copy ldn name 11-12
			strcpy(&w_buffer[13],(unsigned char *) W1_BUFFER_ADR);		//append old buffer
			continue;
		}
		
		if( (entry[11] & (ATTR_HIDDEN | ATTR_VOLUME)) != 0)	
		{
			*w_buffer = 0x00;
			continue;
		}

		if( *w_buffer == 0x00)								//long filename check
		{	strncpy(w_buffer, entry,11);					//copy Dos Name 
		}

		{	unsigned long * l_ptr = (unsigned long *) &DirBuffer[SaveOffset];
			l_ptr[0] = (*(unsigned long*)&entry[20] <<16) | *(unsigned int*)&entry[26]; //save Cluster
			l_ptr[1] = *(unsigned long*) &entry[28];			//save size

			if (strcmp((unsigned char *) W_BUFFER_ADR,"playlist.m3u") == 0)		//compare with name "playlist.m3u"
			{	PlaylistStartCluster = l_ptr[0];				//names are identical -> save start
				PlaylistLaenge =  l_ptr[1];						//save length
				continue;										//next please!
			}			
		}			
/*		
		if ((w_buffer[0] > 0x39) || (w_buffer[0] < 0x30) ||
			(w_buffer[1] > 0x39) || (w_buffer[1] < 0x30)) 
		{	
			sendstring("\n\r? ->");		
			sendstring(w_buffer);		
			return 2;
		}
*/
		TrackNummer = (w_buffer[0] - 0x30) *10 + w_buffer[1] - 0x31;		

		NameBuffer[TrackNummer] = SaveOffset;		   	//save in name buffer start address of entry
		SaveOffset += (2* sizeof(unsigned long));																				
		
		if((entry[11] & ATTR_DIR) == ATTR_DIR)			//if directory:
		{  	
			Start = 2;									//hide first 2 characters (the number)
			w_buffer[2] = DIR_MARK;						//mark directory
		}
		else 
		{	w_buffer[strlen(w_buffer)-4] = 0;					// hide extension .mp3 
			Start = 0;
		}
		strcpy( &DirBuffer[SaveOffset], &w_buffer[Start]);		//save filename
		SaveOffset += strlen(w_buffer) +1;						//+1 for 0
		*w_buffer = 0x00;										//close with 0
		DirCount++;												//increase number of entries
	}
	return 0;		//continue with next sector, directory is not finished yet
}

/***********************************************************************************
- Startet das Einlesen einen Verzeichnisses
- liest alle Sektoren eines Verzeichnisseintrages
- starts to read one directory
- reads all sectors of a dir
***********************************************************************************/
void dir( unsigned long cluster)
{
	unsigned long sec_to_read;							//how many sectors?
	unsigned long first_sector;							//which is the first?

	DirCount = 0;										// init DirCount
	SaveOffset = 0;		
	PlaylistLaenge = 0;
	*(unsigned char *)W_BUFFER_ADR =0;
	memset((unsigned char *)NAMEBUFFER_ADR, 0x00, NAMEBUF_S);	//set namebuffer to zero

	if (cluster == 0)										
	{
		first_sector = DIR_START;						//read ROOT dir
		sec_to_read = Fat32Length;
	}
	else
	{
		first_sector = cluster_to_sector( cluster);		//read subdir
		sec_to_read = ClusterSize;
	}
	
	while (TimeWindow == 0);							//waiting until update of playbuffer is finished
	while(1)
	{	
		unsigned long sector;
		for(sector = first_sector; sector < sec_to_read + first_sector; sector++)
		{
			if  ( read_fat_sector(sector))	return ;
		}		
		cluster = get_next_cluster(cluster);			 
		first_sector = cluster_to_sector(cluster);
	}
}

/***********************************************************************************
- berechnet die erste Sectornummer aus der Clusternummer
***********************************************************************************/
unsigned long cluster_to_sector( unsigned long cluster)
{
//	return  DataStart + (cluster-2) * ClusterSize;
	return  DataStart + ((cluster-2) <<4);
}

/***********************************************************************************
- ein Sektor der Fat wird gelesen und in den FatBuffer geschrieben
- update fat buffer: reads one sector of fat and put it into fat buffer
***********************************************************************************/
#define FAT32_ENTRYS_SECTOR	(SECTORSIZE / 4)
void get_fat( unsigned long cluster)
{
	read_lba( FAT_START + (cluster>>7), (FAT_BUF_S/SECTORSIZE), (unsigned char *)FATBUFFER_ADR);
}

/***********************************************************************************
- es wird aus der Clusterkette der nchste Cluster gesucht ber die FAT
- wenn der Cluster nicht im FatBuffer steht, wird der ensprechende Teil aus der Fat geladen
- looking for the next cluster in cluster chain
- if the cluster can not be found, the fat buffer will be updated
***********************************************************************************/

#define ENTRY_CACHE_32	(FAT_BUF_S / 4)

unsigned long get_next_cluster( unsigned long prev_cluster)
{
unsigned long *FatBuffer	= (unsigned long *)	FATBUFFER_ADR;

	if	((prev_cluster >= (ENTRY_CACHE_32 + (first_cluster_in_cache)))||	//cluster present in fat buffer?
		(prev_cluster <  first_cluster_in_cache ))
	{
		get_fat( prev_cluster);												//no: update necessary
		first_cluster_in_cache = prev_cluster & 0xffffff80;
	}
	return (FatBuffer[prev_cluster - first_cluster_in_cache]);
}

/***********************************************************************************
- es wird ein Cluster eingelesen und in den Playbuffer geschrieben
- update the playbuffer: reads one cluster (last_cluster) and write it into playbuffer
***********************************************************************************/
void update_buf(unsigned char *buffer )
{
	read_lba(cluster_to_sector(last_cluster),ClusterSize, buffer);
	last_cluster = get_next_cluster(last_cluster);

/*	if (ClusterSize == 8)
	{	read_lba(cluster_to_sector(last_cluster),ClusterSize, &buffer[0x1000]);
		last_cluster = get_next_cluster(last_cluster);
	}
*/	
	TimeWindow = 1;			//this flag shows, that the playbuffer was updated, it will be cleared in timerint
}

















							   