
#include "my_defs.h"
#include "memory.h"
#include "serial.h"
#include "tools.h"
#include "progmem.h"
						   
#include "ide.h"
#include "fat.h"
#include "i2c.h"
#include "mas.h"
#include "lcd.h"
#include "rc5.h"
#include "debug.h"


// Definition des Timer1 Reload Wertes
#define T1PERIODE	0.004F	// Periode des Timer1-Interrupts in ms
#define T1RELOAD	(0x10000 - ((F_CPU / 8) * T1PERIODE))
#define TI1_H		((unsigned short) T1RELOAD >> 8)
#define TI1_L		((unsigned short) T1RELOAD & 0xff)


unsigned char Entry =-1;			//Nummer des aktuell eingestellten Liedes (0..99)== die ersten 2 Zeichen
									// des Dateinamens -1
unsigned char *ptr_bytes_out;	    //Ptr auf das letzte an den MAS ausgegebende Byte

unsigned char ST_REG;				//hier ist immer der aktuelle Playstatus drin
unsigned char SinglePlay = FALSE;	//TRUE: nach dem spielenden Lied ist Stop, sonst nchstes Lied
unsigned char NextSongUp;			//TRUE: Nach dem spielenden Lied wird das nchste Leid gespielt

unsigned char UPDATE_BUF;			//zeigt an, welcher Play-Buffer neu gefllt werden mu

unsigned long file_len;				//Lnge der gerade spielenden Datei in Byte
unsigned long bytes_play;			//Anzahl der Bytes, die schon gespielt wurden

volatile unsigned char Time;		//wird im Timerint hochgezhlt
volatile unsigned char TimeWindow;	//zeigt an, da ein auerplanmiger Plattenzugriff erfolgen darf, ohne da es Aussetzer gibt

unsigned long * get_song(void);		//gibt einen Zeiger zurck auf eine "spezielle" Struktur im DirBuffer
void inline Show_play_title(void);	//gibt Dirname und Songname auf der LCD aus
void get_next_song(unsigned char);	//holt den nchsten oder vorhergehenden Song
unsigned char* name(void);			// gibt einen Zeiger auf den Song - oder Dirnamen zurck
void DisplayLine(unsigned char, unsigned char*);									   
void inline play_file(unsigned long*);
void PumpData(void);				// Pumpt die Daten zum MAS
void WriteLS(void);					// sendet die Lautstrke an den MAS uns peichert im EEprom
unsigned char GetNextBytePlaylist(void);	//gibt das nchste Byte aus der Playlist zurck
void inline SendPlaylist(void);
void inline SendTag(void);
void SavePosition(unsigned char);
void RestorePosition (unsigned char);

unsigned long ActualPath;			// Cluster des aktuellen Dir
unsigned char Deep =0;				// aktuelle Tiefe der  Unterverzeichnisse

unsigned char Loudness;				// Lautstrke fr den MAS
unsigned char Flash = FALSE;		// LCD flash

#if DEBUG == FALSE
/***********************************************************************************
Der ext. Interrupt 0: hier ist der Demand-Pin vom MAS angeschlossen
Bei steigener Flanke wird der Int ausgelst.
Wenn gerade Waitstates aktiviert sind, werden sie rausgenommen und am Ende wieder 
eingefgt. 
Es werden neue Daten zum MAS geschaufelt.
***********************************************************************************/
#if MINIMEGAHMPEG == FALSE	// Hmpeg
SIGNAL(SIG_INTERRUPT0) //DEMAND
#else	// MiniMegaHmpeg
/* External Interrupt1 Function name */
#define	SIG_INTERRUPT4 _interrupt4_
SIGNAL(SIG_INTERRUPT4) //DEMAND
#endif
{	//ca alle 12 ms fr 2-3ms
//	unsigned char n = inp(MCUCR);		// Waitstates?
//	cbi(MCUCR,SRW);						// no Waitstates!
	PumpData();
//	outp(n,MCUCR);						// evt. Waitstates.
}

/***********************************************************************************
Der Timerint: alle ?ms 
Es wird
- das "Zeitfenster" gelscht, wenn der Player spielt, bzw gesetzt bei STOP 
- geschaut, ob ein Playbuffer leer ist und ggf. eine neue Befllung veranlat.
- der Timer neu geladen
- wenn gebinkt werden soll die LCD ein- und ausgeschaltet
Wenn gerade Waitstates aktiviert sind, werden sie rausgenommen und am Ende wieder 
eingefgt. 
***********************************************************************************/			 				  
SIGNAL(SIG_OVERFLOW1)	//timer 1 overflow jede 2ms
{
//	unsigned char n = inp(MCUCR);		// Waitstates?
//	cbi(MCUCR,SRW);						// no Waitstates!

	// disable HD access if playing
	TimeWindow= (ST_REG & 1 << ST_PLAYING) ? 0 : 1;
/*
	if ((ST_REG & (1<<ST_PLAYING)) == (1<<ST_PLAYING))
	{	if (TimeWindow) TimeWindow--;
	}
	else TimeWindow = 3;
*/										   
	switch(UPDATE_BUF)
	{	case 1:
			update_buf((unsigned char *)PLAY_BUFFER_START);	
		break;
		case 2:
			update_buf((unsigned char *)PLAY_BUFFER_2);
	 	break;
	}
	UPDATE_BUF = 0;

	outp(TI1_H,TCNT1H);					//reload timer 
	outp(TI1_L,TCNT1L);
#if LCD
	if ((Time++ == 0 ) && (Flash == TRUE))
		lcd_command(LCD_COMMAND_OFF);
	else if (Time == 100 )
		lcd_command(LCD_COMMAND_ON);
#endif
	
//	outp(n,MCUCR);						// evt. Waitstates.
}

/***********************************************************************************
Es werden
- der Controller und die Hardware initialisiert
- die Hauptschleife gestartet
***********************************************************************************/
int main( void )
{
unsigned char FirstNumber = 0;
unsigned int rc_cmd, rc_cmd_old = 0;
unsigned char * PtrArgBuff= (unsigned char*) ARG_BUFFER_ADR;
unsigned char ArgumentCount = 0;
unsigned char Command, EnableShowPlayTitel = TRUE;
unsigned char PlayButton = FALSE;

#if SERIAL == TRUE
    outp((0<<RXCIE) |(0<<TXCIE)|(1<<RXEN) |(1<<TXEN),UCR);    	// enable TX and RX
    outp(UART_BAUD_SELECT, UBRR); 								// set baudrate
#endif
	
	
	//diverse Interrupteinstellungen, ext. RAM
#if MINIMEGAHMPEG == FALSE	// Hmpeg
    outp( (1<<SRE) 						// external SRAM enable
    	| (1<<ISC00) | (1<<ISC01) 		// rising edge of INT0 (DEM) generates an irq
    	| (1<< ISC10) | (1<<ISC11)		// rising edge of INT1 generates an irg (not really used as IRQ)
    	, MCUCR);

	timer0_source(CK256);	// Timer0 Clock = F_CPU / 256
#else	// MiniMegaHmpeg
    outp(1<<SRE, MCUCR);
    outp( (1<<ISC41) | (1<<ISC40), EICR);	// rising edge of INT4 (DEM) generates an irq 

	timer2_source(CK256);	// Timer2 Clock = F_CPU / 256
#endif


	i2c_init();								  
	mas_init();
	ide_init();
	lcd_init();
	
	outp((1<<CS11),TCCR1B);	//prescaler CK Timer 1 (F_CPU / 8)
	sbi(TIMSK, TOIE1);	 	//enable timer1 interrupt
#if MINIMEGAHMPEG == FALSE	// Hmpeg
	outp(1<<INT0, GIMSK);	//enable int0
#else	// MiniMegaHmpeg
	outp(1<<INT4, EIMSK);	//enable int4
#endif
    sei();                 /* enable interrupts */
	
//***********************************************************
// MAIN
#if LCD
	lcd_nprint("HD V0.991m2",0xff);
#endif

	init_fat();
	
	dir(0);
	Loudness = eeprom_rb(8);							
	if (Loudness == 0xff) 
	{	Loudness = 0x80;
	}
	WriteLS();	
	*(unsigned int*) DIRNAME_ADR = 0;	
	ActualPath = 0;
	NextSongUp = 1;
	putchar (READY);
	if (eeprom_rb(7))						//Play on Power-UP?
	{	
		ST_REG = 1 << PLAY_ENABLE;
		if (PLAYBUTTON_NOT_PRESSED) 
		{	
			RestorePosition(10); 			// Play button pressed?
		}
		goto PlayNext;
	}														 
	else 
	{	
		goto SetNext;
	}	
/***********************************************************************************
Die Hauptschleife
Es wird
- die FB abgefragt und bedient
- die ser. Schnittstelle abgefragt und bedient
- der Drehschalter auskodiert
- der LCD - Text geschoben
- ein neues Lied gestartet, wenn das letzte zu Ende ist				    
- eigentlich viel zu viel fr eine Funktion
***********************************************************************************/
    while (1)
    {
    	
#if RC5 == TRUE
		if (REMOTE_CLEAR)					//Telegramm von der Fernbedieung?
		{	
			cli();							// Int sperren
			rc_cmd = get_rc5();				// RC5 - Telegramm entschlsseln
			PumpData();						// braucht der MAS neue Daten?
			sei();							// Int wieder zulassen
 			if ((rc_cmd != rc_cmd_old) && (rc_cmd != 0))
			{								// neue Taste auf der FB wurde gedrckt
				rc_cmd_old = rc_cmd;		// Code merken fr den nchsten Vergleich
//				print_long(rc_cmd);new_line();	// Code von der FB anzeigen lassen				
				EnableShowPlayTitel = TRUE;
				switch((unsigned char)rc_cmd)	// Gerteadresse ist ausgeblendet
				{
					case IR_SETNEXT:
						goto SetNext;						
					break;
					case IR_SETPREV:
						goto SetPrev;
					case IR_VOLUME_DOWN:
						if (Loudness <= 0xFF-0x08)
						{	Loudness +=0x08;						
							WriteLS();
						}
					break;					
					case IR_VOLUME_UP:
						if (Loudness >= 0x80 + 0x08)
						{	Loudness -=0x08;
							WriteLS();
						}
					break;					
					case IR_STOP:		goto Stop;	
					case IR_PLAYNEXT:  	goto PlayNext;
					case IR_PROG_UP:	goto NextDir;	
					case IR_PROG_DOWN:	goto PrevDir;	
					case IR_SAVE:		goto Save;	
					case IR_RESTORE:	goto Resume;	
					case IR_PAUSE:		goto Pause;	
					case IR_SHOWNEXT:	goto ShowNext;	
					case IR_10PLUS:		FirstNumber = 10;	
					break;
					case IR_20PLUS:		FirstNumber = 20;	
					break;
				}
				if ((unsigned char) rc_cmd < IR_9)
				{	Entry = FirstNumber + rc_cmd -1;
					FirstNumber = 0;
					NextSongUp = 0;
				 	goto Play;
				}
			}
		}
#endif
#if SERIAL == TRUE				
		if (bit_is_set(USR,RXC))		//ein Byte von der RS232 empfangen worden?
		{
			unsigned char Echo = inp(UDR);	
			unsigned char EchoAlreadySend = FALSE;	//so ein Dreck!

			if (ArgumentCount == 0)		//Argument oder Befehl?
			{				
				switch (Echo)			// je nach Befehl die Anzahl der Bytes des Arguments laden
				{						// die Befehle, die hier nicht stehen, haben kein Argument
					case READ_REG_MAS:		// unsigned char REG
					case WRITE_STATUS:
						ArgumentCount =	1;	// erwartet eine Zahl zwischen 0 und 99
					break;
					case DIR_CLUSTER:		// unsigned long Clusternummer
					case READ_MEM_MAS:		// unsigned char MEM, unsigned int ADR,0
					case RUN_ADR_MAS:		// unsigned int ADR,0,0
						ArgumentCount =	4;
					break;
					case PLAY_ENTRY:
					case WRITE_MEM_MAS:		// unsigned char MEM, unsigned int ADR, unsigned long data,0
					case WRITE_REG_MAS:		// unsigned char REG, unsigned long data,0,0,0					
						ArgumentCount =	8;
					break;					
					case GO:				// als Abschlu kam da GO! - Kommando, also:
						PtrArgBuff = (unsigned char*) ARG_BUFFER_ADR;	//Nullsetzen vom Zeiger aufs erste Arg
						Echo = 0x00;		// wenn Befehl erkannt wird, ist die Null das Echo
#endif
						switch(Command)
						{	
SetNext://					if (ST_REG == 1 << ST_PLAYING) ST_REG = 1 << PLAY_ENABLE;
							case SET_NEXT:
								get_next_song('+');
							break;							
							case PLAY_NEXT:	//skip current song, continue with next
PlayNext:						ST_REG = 1 << PLAY_ENABLE;
							case SHOW_NEXT:
ShowNext:						if (NextSongUp) get_next_song('+');
#if LCD
							   	Flash = TRUE;
#endif								
							break;
							case PLAY_PREV:
								ST_REG = 1 << PLAY_ENABLE;								
SetPrev:  //						if (ST_REG == 1 << ST_PLAYING) ST_REG = 1 << PLAY_ENABLE;
							case SET_PREV:
								get_next_song('-');
					  		break;
NextDir:					   
							case NEXT_DIR:
								do
								{	get_next_song('+');	
								}	while (Entry !=0);
							break;
PrevDir:					   
							case PREV_DIR:	  
								while (Entry !=0) get_next_song('-');
								do
								{	get_next_song('-');					
								}	while (Entry !=0);
							break;
					
							case PLAY_STOP:
Stop:							ST_REG = STOP;
							break;

							case PAUSE:
Pause:							if (ST_REG == 1 << ST_PLAYING)
								{	
									ST_REG = 1 << PAUSE;
//									lcd_command(LCD_COMMAND_CLS);
//									DisplayLine(LCD_ADR_L0 + (LCD_SPALTEN / 2) -3, "PAUSE");
//									EnableShowPlayTitel = FALSE;
#if LCD
									Flash = TRUE;
#endif									
								}								
								else if (ST_REG == 1 << PAUSE)
								{
									ST_REG = 1 << ST_PLAYING;
									PumpData();						// braucht der MAS neue Daten?
#if LCD									
									Flash = FALSE;
#endif									
								}
							break;
Resume:						
#if LCD
								lcd_command(LCD_COMMAND_CLS);
								DisplayLine(LCD_ADR_L0 + LCD_SPALTEN / 2-3, "Wait...");								
								Flash = TRUE;
#endif								
							case RESUME:
								RestorePosition(0);
							break;			

							case SAVE:					//speicher den akt. Titel im EEprom
Save:							SavePosition(0);
							break;

							case PLAY:
Play: 							if (Entry >= DirCount) Entry = DirCount-1;
								ST_REG = 1 << PLAY_ENABLE;
								NextSongUp = 0;
							break;				
#if SERIAL == TRUE				
							case PLAY_ENTRY:
								Entry = PtrArgBuff[0];
								ActualPath = *(unsigned long*) &PtrArgBuff[1];
								dir (ActualPath);
								NextSongUp = 0;
								goto Play;
							break;						
							case INFO:
							{
								unsigned char n=0;
								unsigned int *NameBuffer = (unsigned int *) NAMEBUFFER_ADR;
								unsigned char *DirBuffer = (unsigned char *) 	DIRBUFFER_ADR;

								putchar (0);			//Echo first
								EchoAlreadySend = TRUE;	//so ein Dreck!
								sendstring(name()); 	new_line();
								sendstring((unsigned char *)DIRNAME_ADR); 	new_line();
								print_long(DirCount);	new_line();
								while (n < DirCount)
								{	
									print_long(*(unsigned long*) &DirBuffer[NameBuffer[n]]); 
									print_long(*(unsigned long*) &DirBuffer[NameBuffer[n] + sizeof(unsigned long)]); 
									sendstring ((unsigned char*) &DirBuffer[NameBuffer[n++] + 2*sizeof(unsigned long)]);
									new_line();
								}
							}
							break;
							case SEND_ID3TAG:
								putchar (0);				//Echo first
								EchoAlreadySend = TRUE;		//so ein Dreck!
								SendTag();
							break;					

							case SEND_PLAYLIST:
								EchoAlreadySend = TRUE;		//so ein Dreck!
								putchar (0);				//Echo first
								SendPlaylist();								
							break;
							
							case DIR_CLUSTER:
								ActualPath = *(unsigned long*) PtrArgBuff;
								dir (ActualPath);
							break;						

							//Fr den MAS
							case READ_REG_MAS:				// unsigned char REG
								EchoAlreadySend = TRUE;		// so ein Dreck!
								putchar (0);				// erstmal das Echo
								print_long( read_reg( PtrArgBuff[0]));
								new_line();
							break;					
					
							case READ_MEM_MAS:				// MEM, unsigned int ADR, 0
								EchoAlreadySend = TRUE;		//so ein Dreck!
								putchar (0);				//erstmal das Echo
								print_long( mas_read_mem( PtrArgBuff[0], *(unsigned short*) &PtrArgBuff[1]));
								new_line();
							break;					
					
							case RUN_ADR_MAS:				// unsigned int ADR,0,0
								mas_run( *(unsigned int*)PtrArgBuff);
							break;
					
							case WRITE_MEM_MAS:				// MEM, unsigned int ADR, unsigned long data, 0
								mas_write_mem(PtrArgBuff[0],*(unsigned int*) &PtrArgBuff[1], *(unsigned long*) &PtrArgBuff[3]);	
							break;					
					
							case WRITE_REG_MAS:				// unsigned char REG, unsigned long data,0,0,0					
								write_reg(PtrArgBuff[0],*(unsigned long*) &PtrArgBuff[1]);						
							break;
							case READ_DEFAULT:
								EchoAlreadySend = TRUE;		//so ein Dreck!
								putchar (0);				//erstmal das Echo
								print_long(default_read());
								new_line();
							break;
							case NOP:
							break;
							case SEND_STATUS:														   
								EchoAlreadySend = TRUE;		//so ein Dreck!
								putchar (0);				//erstmal das Echo
								putchar (((ST_REG == 1<<ST_PLAYING) <<2) | (SinglePlay <<1) | (eeprom_rb(7) & 0x01));
							break;
							case WRITE_STATUS:
								SinglePlay = ((PtrArgBuff[0] & 2) == 2);
								if (PtrArgBuff[0] & 1) eeprom_wb(7, 1);
							break;								
							default: 
								Echo = 0x01;	//Befehl erhalten, aber nicht bekannt
							break;
#endif														
						}				
#if SERIAL == TRUE											
					break;
				}
				Command = Echo;					//Befehl speichern
			}
			else 								//kein Befehl, sondern Argument
			{	*(PtrArgBuff++) = Echo;		//save argument
				ArgumentCount--;				//wieder einer weniger!				
			}
			if (EchoAlreadySend == FALSE) 
				putchar (Echo);					//send echo
		}
#endif		
		if (ST_REG  == 1 << PLAY_ENABLE)		//start new song
		{
			if (NextSongUp) get_next_song('+');
			play_file(get_song());				// start play
			NextSongUp = TRUE;
#if LCD			
			EnableShowPlayTitel = TRUE;	
			Flash = FALSE;
#endif			
			SavePosition(10);					// save position in eeprom for resume after power up
		}
#if LCD
#if SCROLL == TRUE
		// LCD Shift ?
		if ((Time ==0) && (ST_REG  != 1 << PAUSE) &&
		   ((strlen(name()) > LCD_SPALTEN) || (strlen((unsigned char *)DIRNAME_ADR) > LCD_SPALTEN)))
		{	
			lcd_command(LCD_COMMAND_SHIFT);
			Time = 0xa0;					// Scroll speed, increase it to make scrolling faster
		}
#endif
		if (EnableShowPlayTitel)			// display song?
		{	
			EnableShowPlayTitel = FALSE;	// display only once, else it can't scroll
			Show_play_title();
			Time =1;						// set time to next croll of display
		}
#endif

#if DREHSCHALTER == TRUE
		{
			unsigned char static Direction = 128;	   
			unsigned char static LastStateA;
			unsigned char static LastStateB;
			unsigned char static EqualCounter;

			if ((TRACK_A != LastStateA) && (TRACK_B ==  LastStateB))
			{	//true, if signal A is different to last state and signal B is equal to last
				if (Direction < 255) Direction ++;
			}
			else if ((TRACK_B != LastStateB) && (TRACK_A ==  LastStateA))
			{	//true, if signal B is different to last state and signal A is equal to last
				if (Direction > 0) Direction --;
			}
			
			if (TRACK_A == TRACK_B)
			{	
				if (!EqualCounter++)			// A and B have to be equal for a longer period!
				{	
					LastStateA = TRACK_A;		//store last state A
					LastStateB = TRACK_B;		//store last state B
					if (Direction != 128)		//Direction changed?
					{
						EnableShowPlayTitel = TRUE;	
						if (Direction > 140)
						{
							Direction = 128;
							goto SetNext;
						}
						if (Direction < 110)
						{
							Direction = 128;
							goto SetPrev;
						}
						Direction = 128;
					}
				}
			}
			if (PLAYBUTTON_PRESSED)				// Play button pressed?
			{	
				if (PlayButton == FALSE)		// debounce - entprellen
				{	PlayButton = TRUE;
					goto PlayNext;
				}
			}
			else if (Time > 10) 				// debounce time 40ms
				PlayButton = FALSE;
		}
#endif		
  	}	//Ende von while (1)
}									  
#endif
/***********************************************************************************
Der Titel wird angezeigt, der sich hinter "Entry" verbirgt. Das ist entweder der 
gerade gespielte oder wenn durch Bedienung ein neuer Titel ausgesucht wurde, dann dieser.
***********************************************************************************/
#if LCD
void inline Show_play_title(void)
{
	lcd_command(LCD_COMMAND_CLS);
	DisplayLine(LCD_ADR_L0, name());
	DisplayLine(LCD_ADR_L1, (unsigned char *)(DIRNAME_ADR +1));
}

/***********************************************************************************
Zeigt die ersten 40 Zeichen von "*Text" an der Position "LcdLine" an.
Wenn Text mehr als 37 Zeichen gro ist, wird ".." angehngt.
***********************************************************************************/
void DisplayLine(unsigned char LcdLine, unsigned char* Text)
{
	lcd_command(LcdLine);	
	lcd_nprint(Text,40);	
#if LCD_SPALTEN < 37
	if (strlen(Text) > 37)
	{	lcd_command(LcdLine + 37);	
		lcd_nprint(".. ",0xff);
	}
#endif		
}
#endif
/***********************************************************************************
Je nach Richtung wird "Entry" entweder auf den nchsten oder der vorhergehenden Titel 
gesetzt. Dabei wird in Unterverzeichnisse gewechselt und er Name der Verzeichniskette
nach DirName kopiert. Nach dem letzten Lied auf der Platte wird bei Richtung '+' das 
erste Leid der Platte zurckgegeben. Nach dem ersten Lied auf der Platte wird bei 
Richtung '-' das letzte Lied im letzten Verzeichnis zurckgegeben.
Nach dem Aufruf steht in DirCount die Summe aus Verzeichnissen und Titeln des 
Verzeichnisses. "ActualPath" hat danach den Startcluster des aktuellen Verzeichnisses.

***********************************************************************************/
void get_next_song(unsigned char Direction)
{
#define MaxPathDeep 8
static unsigned char LastEntry[MaxPathDeep];
static unsigned long LastDir[MaxPathDeep];
static unsigned char LastDirEnde[MaxPathDeep];
unsigned char* DirName = (unsigned char *)DIRNAME_ADR;

	NextSongUp = 0;								//merken, da die Titelreihenfolge 
												//gendert wurde
	while (1)
	{	if (Direction == '-')		
		{	if (Entry == 0xff) Entry = DirCount; //Um den letzten Eintrag zu bekommen
			Entry--;
		}
		else	
		{	Entry++;
		}

		if (Entry < DirCount)					//nchsten Eintrag bearbeiten
		{	if (*name() == DIR_MARK)			//es handelt sich um ein Dir!
			{	LastDir[Deep] = ActualPath;		//merken fr die Rckkehr
				LastDirEnde[Deep] = strlen(DirName);	//merken fr die Rckkehr
				strcat(DirName,name());			//ranhngen an den alten Namen
				LastEntry[Deep++] = Entry;		//hier haben wir aufgehrt
				ActualPath = *get_song();		//hier jetzt weitermachen Dir
				dir(ActualPath);				//Dir einlesen
				Entry = -1;						//und hier gehts weiter	File
			}
			else 								//ok, nchter Titel ist gefunden
			{	if (Deep == 0) DirName[1] =0;	//wir sind im Rootverzeichnis-> kein Name
				return ;						//fertig.
			}
		}
		else 
		{	if (Deep !=0)
			{
				ActualPath = LastDir[--Deep];	//das war der letzte Pfad
				dir(ActualPath);				//letztes Verzeichnis reinholen			
				Entry = LastEntry[Deep];		//hier hatten wir aufgehrt
				DirName[LastDirEnde[Deep]] =0; 	//letzten Pfad aus dem Namen rausschmeien
			}
			else 								//Plattenende erreicht
			{	
				if ((SlavePresent) && (ST_REG != 1 << ST_PLAYING))
				{	DEVICE = (DEVICE == CB_DH_DEV0) ? CB_DH_DEV1 : CB_DH_DEV0;
					delay_ms(100);
					init_fat();	  				//nchste Platte
				}
				ActualPath = 0;					//das ist der erste Pfad
				dir(ActualPath);				//Root reinholen		
				Entry = -1;						//Wird weiter oben incrementiert
				*DirName =0; 
			}
		}
	}
}						 

/***********************************************************************************
Gibt einen Zeiger auf den Namen zurck, auf den "Entry" zeigt
Wenn das erste Zeichen des Namens ein "DIR_MARK" ist, handelt es sich um ein Verzeichnis.
***********************************************************************************/
unsigned char* name(void)
{
unsigned int *NameBuffer = (unsigned int *)  NAMEBUFFER_ADR;
unsigned char *DirBuffer = (unsigned char *) DIRBUFFER_ADR;
	return ( &DirBuffer[NameBuffer[Entry] + 2 * sizeof(unsigned long)]);
}

/***********************************************************************************
Gibt einen Zeiger auf einen Struct zurck, der zu Entry gehrt.
struct
{	unsigned long StartCluster;
	unsigned long Laenge;
	unsigned char Name[?];	//0-terminiert	
}
***********************************************************************************/
unsigned long * get_song(void)
{
unsigned int *NameBuffer = (unsigned int *)	NAMEBUFFER_ADR;
unsigned char *DirBuffer = (unsigned char *) DIRBUFFER_ADR;
	return ( (unsigned long*) &DirBuffer[NameBuffer[Entry]]);
}

/***********************************************************************************
- fllt zuerst die Playbuffer 1 und 2 auf.
- setzt den Zeiger "ptr_bytes_out" fr das nchste auszugebende Byte auf das erste 
  Byte im Playbuffer 1
- setzt den Zhler "bytes_play" fr die gespielten Bytes auf Null
- setzt das Statusregister auf "ST_PLAYING"
- lt die ersten Bytes zum MAS pumpen 
***********************************************************************************/
void play_file(unsigned long *entry_num)
{
	last_cluster = entry_num[0];		//set first cluster
	file_len = entry_num[1];			//set file length
	update_buf((unsigned char *)PLAY_BUFFER_START);	//update first and 2nd. playbuffer
	update_buf ((unsigned char *)PLAY_BUFFER_2);
	ptr_bytes_out = (unsigned char *)PLAY_BUFFER_START;	//set pointer to start of playbuffer
	ST_REG = 1<<ST_PLAYING;				//set status
	bytes_play = 0; 					//how many bytes are played? 0!
	PumpData();							//pump data to MAS3507
}

/***********************************************************************************
Die Funktion sendet die Daten zum MAS. Dazu wird zuerst Playbuffer 1 ausgelesen, wenn 
dieser "leer" ist wird das Flag UPDATE_BUF auf 1 gesetzt und der Playbuffer 2 ausgelesen. 
Im Timerint wird das Flag "UPDATE_BUF" abgefragt und ggf. der entsprechende Playbuffer gefllt. 
Wenn Daten von der FB eintreffen oder der DEMAND-PIN vom MAS low ist, wird die Funktion 
verlassen. Wenn die Datei ausgelesen wurde, wir das Statusregister ST_REG auf Stop oder 
Play_Enable gesetzt, jenachdem ob der Spielmodus Single ist oder Continues.
***********************************************************************************/
void PumpData(void)				   
{
	if( !(ST_REG & 1<<ST_PLAYING)) return;							//nothing to do here

	while ( DEMAND_SET )	//more Data?
	{	
		if (ptr_bytes_out == (unsigned char *)PLAY_BUFFER_2)		// update of playbuffer 1?
			UPDATE_BUF = 1; //flag setzen							// YES!
		else if (ptr_bytes_out == (unsigned char *)PLAY_BUFFER_END)	// update of playbuffer 2?
		{	ptr_bytes_out = (unsigned char *)PLAY_BUFFER_START;		// continue with playbuffer 1
			UPDATE_BUF = 2; 										// YES!
		}		
		outp(*(ptr_bytes_out++), SPDR);		   						// send data to MAS
				
		if( ++bytes_play >= file_len)								// file length 
		{	
			ST_REG = SinglePlay ? STOP :( 1 << PLAY_ENABLE);		// diese Zeile ist optisch ein Genu!
			break;													// ready, next file please
		}
#if RC5 == TRUE
		if (REMOTE_CLEAR) break;									// return, if remote control (dangerous...)
#endif		
	}
}

/***********************************************************************************
Die Funktion schreibt die aktuell eingestellte Lautstrke in die Register des MAS
und fallse notwendig in die EEpromzelle 8
load sound intensity into register of MAS and if necessary into eeprom cell
***********************************************************************************/
void WriteLS(void)
{	
	unsigned long ls = (unsigned long) Loudness << 12;
	mas_write_mem (MEM_D1, 0x7F8, ls);
	mas_write_mem (MEM_D1, 0x7Fb, ls);							
	if (eeprom_rb(8) != Loudness)
		eeprom_wb(8, Loudness);							
}							

/***********************************************************************************
Die Funktion liest die Playliste aus und gibt bei jedem Aufruf immer das nchste Byte 
der Datei playlist.m3u zurck.
***********************************************************************************/

unsigned char GetNextBytePlaylist(void)			 
{

unsigned char *SectorBuffer	= (unsigned char *) SECTORBUFFER_ADR;
static unsigned long cluster;
static unsigned long ReadBytes =0;
static unsigned char ReadSectors =0;

	if (ReadBytes ==0) 
		cluster = PlaylistStartCluster;
	while (ReadBytes < PlaylistLaenge)		//jedes Byte einzeln
	{											//Dateilnge noch nicht erreicht						
		if ((ReadBytes & 0x1ff)  == 0)		//alle 512 Byte ein Sektor weiter
		{										//n: Index fr Soktorbuffer
			while (TimeWindow ==0);	
			if  (ReadSectors  == 16)		//alle 16 Sektoren ein Cluster weiter
			{
				cluster = get_next_cluster(cluster);
				ReadSectors  = 0;													
			}													
			read_lba(cluster_to_sector(cluster) + ReadSectors++ , 1, SectorBuffer);
		}
		return (SectorBuffer[ReadBytes++ & 0x1ff]);
	}
	ReadBytes =0;
	ReadSectors =0;
	cluster = PlaylistStartCluster;	
	return 0x04;	
}								

/***********************************************************************************
Die Funktion liest die Playliste aus und sendet sie an die serielle Schnittstelle 
Am Ende wird EOT (0x04) gesendet
send file "playlist.m3u" to serial interface, finish with 0x04 (EOT)
***********************************************************************************/
void inline SendPlaylist(void)
{

unsigned char *SectorBuffer	= (unsigned char *) SECTORBUFFER_ADR;
unsigned long cluster;
unsigned long ReadBytes =0;
unsigned char ReadSectors =0;

	cluster = PlaylistStartCluster;				// start cluster 
	while (ReadBytes < PlaylistLaenge)			// job done?
	{											// no, file length not reached yet
		if (!(ReadBytes & 0x1ff))				// every 512 byte next sector
		{										
			while (TimeWindow ==0);				// waiting until play buffer is updated
			if  (ReadSectors  == 16)			// every 16 sectors next cluster
			{
				cluster = get_next_cluster(cluster);	//search for next cluster in cluster chain
				ReadSectors  = 0;													
			}													
			read_lba(cluster_to_sector(cluster) + ReadSectors++ , 1, SectorBuffer);
		}
		putchar (SectorBuffer[ReadBytes++ & 0x1ff]);
	}
	putchar (0x04);	
}								


/***********************************************************************************
Die Funktion sendet vom aktuellen Lied (Entry) die ersten 4 und die letzten 128 Byte
***********************************************************************************/
void inline SendTag(void)
{	

	unsigned char *SectorBuffer	= (unsigned char *) SECTORBUFFER_ADR;
	unsigned long cluster,ReadBytes;
	unsigned int n;
	unsigned char ReadSectors;
	unsigned long *lptr;
	lptr = get_song();			//Pointer auf Eintrag im DirBuffer
	cluster = lptr[0];			//Startcluster auslesen
	ReadBytes = 0;			
	ReadSectors =0;
	while (TimeWindow ==0);
	read_lba(cluster_to_sector(cluster), 1, SectorBuffer);
	for (n = 0; n < 4; n++)
	{	putchar (SectorBuffer[n]);
	}
	for (n = (lptr[1] >> 13)-1; n; n--)			//Berechnung der Anzahl Cluster -1
	{	cluster = get_next_cluster(cluster);	//Clusterkette verfolgen
		ReadBytes+=8192;					
	}
	while (++ReadBytes <= lptr[1])			//ab jetzt jedes Byte einzeln :-(
	{											//Dateilnge noch nicht erreicht						
		if (++n == 512)							//alle 512 Byte ein Sektor weiter
		{	n = 0;								//n: Index fr Soktorbuffer
			if  (++ReadSectors == 16)		//alle 16 Sektoren ein Cluster weiter
			{	while (TimeWindow ==0);	
				cluster = get_next_cluster(cluster);
				ReadSectors = 0;
			}
			if (ReadBytes + 640 >= lptr[1])	// ist der Sektor interessant fr uns?
			{	while (TimeWindow ==0);		//warten
				read_lba(cluster_to_sector(cluster) + ReadSectors , 1, SectorBuffer);
			}
		}
		if  (ReadBytes + 128 >= lptr[1])	//ist das Byte eins der letzten 128?
		{	putchar (SectorBuffer[n]);
		}
	}																	
}														

void SavePosition(unsigned char position)
{
	eeprom_wb(position++, DEVICE);
	eeprom_wb(position++, ActualPath);
	eeprom_wb(position++, ActualPath>>8);
	eeprom_wb(position++, ActualPath>>16);				
	eeprom_wb(position++, ActualPath>>24);									
	eeprom_wb(position++, Entry);				
}

void RestorePosition (unsigned char position)
{
unsigned char tmp = eeprom_rb(position);

	if (tmp != 0xff)
	{	
		if (tmp != DEVICE)
		{	
			ST_REG = STOP;
			delay_ms(100);
			DEVICE = tmp;
			init_fat();
			ActualPath = 0;					//das ist der erste Pfad
			dir (ActualPath);
		}
		while (ActualPath != (eeprom_rw(position +1) | (unsigned long)eeprom_rw(position+3) << 16))
			get_next_song('+');
		Entry = eeprom_rb(position+5);
		NextSongUp = 0;		
		ST_REG = 1 << PLAY_ENABLE;
	}
}
