#include <io.h>
#include <interrupt.h>
#include <signal.h>
#include <eeprom.h>
#include <string.h>

#include "my_defs.h"
#include "memory_x.h"
#if SERIAL == TRUE
	#include "serial.h"
#endif
#include "tools.h"

#include "ide.h"
#include "fat.h"
#include "i2c.h"
#include "mas.h"
#include "lcd.h"
#include "rc5.h"
#include "hmpeg_cmd.h"
//#include "frame.h"

//timer 1 defs
// #define T1_CLK	0x0FFF
//#define TI1_H	((0xFFFF - T1_CLK) >>8)
//#define TI1_L	(0xFFFF -T1_CLK)
#define TI1_H		0xFF
#define TI1_L		0x00
#define __IR		TRUE


#if SERIAL == TRUE
/* serial*/
volatile char *snd_ptr=0x0000;
volatile char snd_count=0;
volatile unsigned char rec_count = 0x00;
#endif

volatile unsigned char ERRNO;
volatile unsigned short song_num;


volatile unsigned char *ptr_bytes_out;
volatile unsigned char ST_REG;
volatile unsigned char UPDATE_BUF;

volatile unsigned long file_len;

extern unsigned short DirCount;
extern unsigned short NumDirs;
extern unsigned short NumFiles;

//nur zweizeilige Displays
#define LCD_ADR_L0		0x00
#define LCD_ADR_L1		0x40

//RC5
extern unsigned char rc_adr;
extern unsigned char rc_cmd;
volatile unsigned char old_adr=0;
volatile unsigned char new_ir=0;

unsigned long * get_song( unsigned short song_num);
void print_song_name( unsigned char * name);
void show_song_tmp(unsigned short song_num_tmp);
unsigned long * show_play_title(void);
void get_next_song(void);
void get_prev_song(void);
unsigned char is_entry_dir(unsigned long *entry);
unsigned char check_crc(void);
void calc_crc(void );

//DEMAND
SIGNAL(SIG_INTERRUPT0)
{
	unsigned char *ptr;					 
	
	if( (ST_REG & (1<<ST_PLAYING)) != (1<<ST_PLAYING) ) return;

	ptr = (unsigned char *)ptr_bytes_out;

	while(1)
	{

		if( ptr == (unsigned char *)PLAY_BUFFER_2)
		{
			//update buffer 1
			UPDATE_BUF = 1; //flag setzen
		}

		if( ptr >= (unsigned char *)PLAY_BUFFER_END)
		{
			//ueberlauf, update buffer 2
			ptr = play_buffer; //an anfang setzen
			UPDATE_BUF = 2; //flag setzen
		}

		//send data

		outp(*ptr, SPDR);
		ptr++;
		*bytes_play=*bytes_play +1;
		loop_until_bit_is_set(SPSR, SPIF);	//wait for end of transmission
		inp(SPSR);

		if( *bytes_play >= file_len)
		{
			ST_REG = (1<<PLAY_ENABLE);
			break;
		}
		//More Data ??
		if( bit_is_clear(PIND, MAS_DEMAND)) break;
	}
	ptr_bytes_out = ptr;
}


//timer 1 overflow
SIGNAL(SIG_OVERFLOW1)
{

//update_buffer:
	switch(UPDATE_BUF)
	{
		case 1:
			update_buf((unsigned char *)PLAY_BUFFER_1);
			break;
		case 2:
			update_buf((unsigned char *)PLAY_BUFFER_2);
			break;
		default:
			break;
	}
	UPDATE_BUF = 0;

	outp((unsigned char)TI1_H,TCNT1H );
	outp((unsigned char)TI1_L,TCNT1L);
	outp((1<<CS10),TCCR1B);	//prescaler CK

}


//********************************************************************************

int main( void )
{

	unsigned short stmp;
	unsigned long *ptr_entry;
	unsigned char *c_ptr;

#if SHOW_TIME == TRUE
	unsigned char c_min;
	unsigned char c_sec;
	unsigned long p_time;
#endif

#define	MAX_TITLE_NUM	5
//	unsigned char title_num[MAX_TITLE_NUM]={0,0,0,0,0};

#if SERIAL == TRUE
    /* enable RxD/TxD and ints  */
    outp((1<<RXCIE) /*|(1<<TXCIE)*/|(1<<RXEN) |(1<<TXEN),UCR);
    outp(UART_BAUD_SELECT, UBRR); /* set baudrate*/

#endif
    outp( (1<<SRE) ,MCUCR);	/*enable external RAM |(1<<SRW)*/
//    outp( _MCU_REG_NOWS ,MCUCR);	/*enable external RAM |(1<<SRW)*/
    sei();                 /* enable interrupts */

	// LCD init
	lcd_init();
//	lcd_clr_display();

	//IDE init
	ide_init();
//	ide_soft_reset();
	ide_hreset();

// I2C init
	i2c_init();

//init MAS
	mas_init();

//init timer 1 -> SERVICE routine
	outp((1<<CS10),TCCR1B);	//prescaler CK
	outp(TI1_H,TCNT1H );
	outp(TI1_L,TCNT1L);

//enable timer1 interrupt
	sbi(TIMSK, TOIE1);
//enable int0
	sbi(GIMSK, INT0);


// END of init

	*CB_DH =  DEVICE;
	*CB_DC = CB_DC_HD15 | CB_DC_NIEN ;


//***********************************************************
// MAIN
/*
	if(check_crc() == 1)
	{
		//eeprom CRC wrong !
init_start:
		lcd_x = 0;
		while(1)
		{
			lcd_clr_display();
			lcd_print("X :");
			lcd_print(byte_to_dec(lcd_x,0));

//init_loop1:
//			loop_until_bit_is_clear(PINB, PINB0);
//
//			if(bit_is_clear( PINB, PINB0) )
//			{
//				get_rc5();
//			}
			rc_cmd=KEY_2;
			switch(rc_cmd)
			{
				case KEY_1:
						lcd_print("16");
						lcd_x = 16;
						break;
				case KEY_2:
						lcd_print("20");
						lcd_x = 20;
						break;
				case KEY_3:
						lcd_print("40");
						lcd_x = 40;
						break;
				case PLAY_NEXT:
						goto init_end;
						break;
			}

		}

init_end:
		eeprom_wb(LCD_X,lcd_x);
		eeprom_wb(LCD_Y,lcd_y);
		calc_crc();

		lcd_y = 0x02;
//		lcd_print("Y:");

	}
	else
	{
		//load machine settings
		lcd_x = eeprom_rb(LCD_X);
		lcd_y = eeprom_rb(LCD_Y);

	}
*/

	lcd_print("Init ");
/*
	for(dummy = 0; dummy < 5; dummy++)
	{
		get_rc5();
		if(rc_cmd == IR_POWER)
		{
			goto init_start;
		}
		delay_ms(100);
	}
*/


	cbi(DDRD, DDB0);

	if(init_fat(DEVICE) != 0)
	{
		//error
		lcd_print("? FAT32");
		while(1);
	}

	//read root dir
	dir(0, DEVICE);
	if(DirCount == 0)
	{
		//Nix eingelesen ?!
		lcd_print("NO FILE");
		while(1);
	}

	ST_REG = 0x00;
	song_num = 0;
	dir_name[0] = 0x00;

//MAIN LOOP
new_ir=1;
rc_cmd=PLAY_NEXT;
rc_adr=0x20;
old_adr=0;
    for (;;)
    {

#if __IR == TRUE
	if(new_ir == 1)
	{
		new_ir = 0;
		switch(rc_cmd)
		{
			case PLAY_NEXT:
song_up:
					get_next_song();
					goto seek_song;
					break;

			case PLAY_PREV:
//song_down:								    
					get_prev_song();
					if(song_num == 0)
					{
						//hmm, nix vorher ? nochmal !
						song_num = DirCount+1;
						get_prev_song();
					}

seek_song:
					ST_REG=0;
					//check if dir !!!
					ptr_entry = get_song(song_num);
					
					if(is_entry_dir(ptr_entry) == 1)
					{
						c_ptr = (unsigned char *)&ptr_entry[2];
						if(  c_ptr[1] != 0x2e)
						{
							//add dirname
							strcat(dir_name, c_ptr);
						}
						else
						{
							//remove string
							stmp = strlen(dir_name);
							stmp--;
							while( dir_name[stmp] != DIR_MARK)
							{
								if(dir_name[stmp] == 0x00) break;
								stmp--;
							}
							dir_name[stmp] = 0x00;
						}
						
						dir(ptr_entry[0], DEVICE);
					}
					//start play
					
					play_file(show_play_title(), DEVICE);
					break;

			case PLAY_STOP:
						ST_REG = 0;			//play stop
						break;
		}//end switch(rc_cmd)


	}//end if(new_ir)

	//next title ?
	if( (ST_REG & ((1<<PLAY_ENABLE) | (1<<ST_PLAYING)) ) == (1<<PLAY_ENABLE) )
	{
		//song ente -> play next song
		goto song_up;
	}


#endif


 }//end of for(;;)
}//end of main


unsigned long * show_play_title(void)
{
	unsigned long *long_ptr;

	lcd_clr_display();

	long_ptr = get_song(song_num);

	print_song_name( (unsigned char *)&long_ptr[2]);

	return long_ptr;
}


//setzt song_num auf nummer des naechsten MPEG eintrags
// song_num= 0 wenn kein naehster eintrag
void get_next_song(void)
{
	unsigned long *long_ptr;

	song_num++;
loopi:
	if(song_num > DirCount)
	{
		song_num=1;
	}

	long_ptr = get_song(song_num);
	if(*(unsigned char *)&long_ptr[2] == DIR_MARK)
	{
		song_num++;
		goto loopi;
	}
}

//setzt song_num auf nummer des vorherigen MPEG eintrags
// song_num= 0 wenn kein eintrag
void get_prev_song(void)
{
	unsigned long *long_ptr;

	//ed gibt keinen eintrag der vor dem ersten oder nullten kommt
	if( song_num < 2)
	{
		song_num = 0;
		return;
	}

	//song_num > 1
	song_num--;
loopi:
	long_ptr = get_song(song_num);
//	if(*(unsigned char *)&long_ptr[2] == DIR_MARK)
	if (is_entry_dir(long_ptr))
	{
		song_num--;
		if(song_num != 0)	goto loopi;
	}
}


//returns 1 if dir; 0 else
unsigned char is_entry_dir(unsigned long *entry)
{
	if(*(unsigned char *)&entry[2] == DIR_MARK) return 1;
	return 0;
}

void print_song_name( unsigned char * name)
{
	unsigned char offset,dummy;
	unsigned char len;
	lcd_print(dir_name);
	lcd_set_addr(LCD_ADR_L1);	
	lcd_print(name);
	
/*	len = strlen(name);
	for( offset=0; offset <len; offset++)
	{
		if(name[offset] == '-') break;
	}


	dummy =offset;
	if(offset > lcd_x  )
	{
		l1_scroll = offset - lcd_x;	//anzahl zuviele zeichen speichern
		dummy = offset - l1_scroll;	//anzahl der anzuzeigenden zeichen
	}
	else
	{
		l1_scroll = 0;
	}

	memcpy(lcd_l1, name, offset); //save line 1
	lcd_l1[offset] = 0x00;
	lcd_nprint( lcd_l1, dummy);	//show line 1


	lcd_set_addr(LCD_ADR_L1);
	if( offset < len)
	{
		//sub title mark - found
		offset++;
		while( name[offset] == ' ') offset++;
		len = len-offset;
		strcpy(lcd_l2, &name[offset]);	//save line 2

		if(len > lcd_x)
		{
			l2_scroll = len - lcd_x;
			len = lcd_x;
		}
		else
		{
			l2_scroll = 0;
		}
		lcd_nprint( lcd_l2, len);
	}
	else
	{
		//anzeige des Paths
		strcpy(lcd_l2, dir_name);		//save lcd line 2

		len = strlen(lcd_l2);
		if(len > lcd_x) len = lcd_x;
		offset = strlen(dir_name) - len;
		lcd_print(&lcd_l2[offset]);
	}
	//clear scroll counter
	scroll_1=0;
	scroll_2=0;
*/
}

//return ptr auf entry num song_num
unsigned long * get_song( unsigned short song_num)
{
	unsigned short tmp;
	unsigned char *tmp1;

	tmp = 1;
	tmp1= dir_buffer;
	while(tmp != song_num)
	{
		tmp1 = tmp1 + 8;
		while( *tmp1 != 0) tmp1++;
		tmp1++;
		tmp++;
	}
	return (unsigned long *)tmp1;
}

//0 if OK
unsigned char check_crc()
{
	unsigned short crc=0;
	unsigned char dummy;

//	cli();
	for(dummy=0; dummy< 253; dummy++)
	{
		crc = crc + eeprom_rb(dummy);
	}

	if(crc != eeprom_rw(SET_CRC))
	{
			sei();
			return 1;
	}
//	sei();
	return 0;
}


void calc_crc()
{
	unsigned short crc=0;
	unsigned char dummy;

//	cli();
	for(dummy=0; dummy< 253; dummy++)
	{
		crc = crc + eeprom_rb(dummy);
	}
	eeprom_wb(SET_CRC+1, (crc>>8));
	eeprom_wb(SET_CRC, crc);
//	sei();

}
