#include <io.h>
#include <interrupt.h>
#include <signal.h>
#include <eeprom.h>
#include <string.h>

#include "my_defs.h"
#include "memory_x.h"
#if SERIAL == TRUE
	#include "serial.h"
#endif
#include "tools.h"

#include "ide.h"
#include "fat.h"
#include "i2c.h"
#include "mas.h"
#include "lcd.h"
#include "rc5.h"
#include "hmpeg_cmd.h"
//#include "frame.h"

//timer 1 defs
// #define T1_CLK	0x0FFF
//#define TI1_H	((0xFFFF - T1_CLK) >>8)
//#define TI1_L	(0xFFFF -T1_CLK)
#define TI1_H		0xFF
#define TI1_L		0x00
#define __IR		TRUE


#if SERIAL == TRUE
/* serial*/
volatile char *snd_ptr=0x0000;
volatile char snd_count=0;
volatile unsigned char rec_count = 0x00;
#endif

volatile unsigned char ERRNO = 0;
volatile unsigned short song_num, song_num_tmp=0;


volatile unsigned char *ptr_bytes_out;
volatile unsigned char ST_REG;
volatile unsigned char UPDATE_BUF=0;

//volatile unsigned long bytes_play;
volatile unsigned long file_len;

volatile unsigned short timer1=0;
//volatile unsigned char drive_att=0;
//extern unsigned char ASC;
//extern unsigned char ASCQ;
extern unsigned short DirCount;
extern unsigned short NumDirs;
extern unsigned short NumFiles;
//LCD

// 8x2 16x2 20x2 40x2
//     16x4 20x4 40x4
//#define	lcd_x	40
//#define lcd_y	2
//nur zweizeilige Displays
#define LCD_ADR_L0		0x00
#define LCD_ADR_L1		0x40

//RC5
extern unsigned char rc_adr;
extern unsigned char rc_cmd;
volatile unsigned char old_adr=0;
volatile unsigned char new_ir=0;

unsigned long * get_song( unsigned short song_num);
void print_song_name( unsigned char * name);
void show_song_tmp(unsigned short song_num_tmp);
unsigned long * show_play_title(void);
void get_next_song(void);
void get_prev_song(void);
unsigned char is_entry_dir(unsigned long *entry);
unsigned char check_crc(void);
void calc_crc(void );

//DEMAND
SIGNAL(SIG_INTERRUPT0)
{
	unsigned char *ptr;
//	unsigned char dummy;

//	if( (ST_REG & (1<<PLAY_ENABLE)) != (1<<PLAY_ENABLE) ) return;
	if( (ST_REG & (1<<ST_PLAYING)) != (1<<ST_PLAYING) ) return;

	ptr = (unsigned char *)ptr_bytes_out;


	while(1)
	{

		if( ptr == (unsigned char *)PLAY_BUFFER_2)
		{
			//update buffer 1
			UPDATE_BUF = 1; //flag setzen
		}

		if( ptr >= (unsigned char *)PLAY_BUFFER_END)
		{
			//ueberlauf, update buffer 2
			ptr = play_buffer; //an anfang setzen
			UPDATE_BUF = 2; //flag setzen
		}

		//send data

		outp(*ptr, SPDR);
		ptr++;
		*bytes_play=*bytes_play +1;
		loop_until_bit_is_set(SPSR, SPIF);	//wait for end of transmission
		inp(SPSR);

		if( *bytes_play >= file_len)
		{
			ST_REG = (1<<PLAY_ENABLE);
			break;
		}
		//More Data ??
		if( bit_is_clear(PIND, MAS_DEMAND)) break;
	}


	ptr_bytes_out = ptr;

}


//timer 1 overflow
SIGNAL(SIG_OVERFLOW1)
{

//update_buffer:
	switch(UPDATE_BUF)
	{
		case 1:
			update_buf((unsigned char *)PLAY_BUFFER_1);
			break;
		case 2:
			update_buf((unsigned char *)PLAY_BUFFER_2);
			break;
		default:
			break;
	}
	UPDATE_BUF = 0;

	outp((unsigned char)TI1_H,TCNT1H );
	outp((unsigned char)TI1_L,TCNT1L);
	outp((1<<CS10),TCCR1B);	//prescaler CK

}


//********************************************************************************

int main( void )
{

	unsigned short stmp;
	unsigned long *ptr_entry;
	unsigned char *c_ptr;


#define	MAX_TITLE_NUM	5
//	unsigned char title_num[MAX_TITLE_NUM]={0,0,0,0,0};

#if SERIAL == TRUE
    /* enable RxD/TxD and ints  */
    outp((1<<RXCIE) |(1<<TXCIE)|(1<<RXEN) |(1<<TXEN),UCR);
    outp(UART_BAUD_SELECT, UBRR); /* set baudrate*/

#endif
    outp( (1<<SRE) ,MCUCR);	/*enable external RAM |(1<<SRW)*/
//    outp( _MCU_REG_NOWS ,MCUCR);	/*enable external RAM |(1<<SRW)*/
    sei();                 /* enable interrupts */

	// LCD init
	lcd_init();
	lcd_clr_display();

	//IDE init
	ide_init();
//	ide_soft_reset();
	ide_hreset();

// I2C init
	i2c_init();

//init MAS
	mas_init();

//init timer 1 -> SERVICE routine
	outp((1<<CS10),TCCR1B);	//prescaler CK
	outp(TI1_H,TCNT1H );
	outp(TI1_L,TCNT1L);

//enable timer1 interrupt
	sbi(TIMSK, TOIE1);
//enable int0
	sbi(GIMSK, INT0);


// END of init

	*CB_DH =  DEVICE;
	*CB_DC = CB_DC_HD15 | CB_DC_NIEN ;


//***********************************************************
// MAIN

	lcd_print("Init");

	cbi(DDRD, DDB0);

	if(init_fat(DEVICE) != 0)
	{
		//error
		lcd_print("1");
		while(1);
	}

	//read root dir
	dir(0, DEVICE);
	if(DirCount == 0)
	{
		//Nix eingelesen ?!
		lcd_print("2");
		while(1);
	}


	ST_REG = 0x00;
	song_num_tmp=1;
	song_num = 0;
	timer1=1;
	dir_name[0] = 0x00;

	lcd_clr_display();
	show_song_tmp(1);

//MAIN LOOP
    for (;;)
    {

#if __IR == TRUE
	if((new_ir == 1) || (rec_count != 0))
	{
		//TODO : check if RC5 addr is for us
		new_ir = 0;
//		sendstring(byte_to_hex(rc_adr) );
//		sendstring(byte_to_hex(rc_cmd) );
//		sendstring("\n\r");

		if (rec_count !=0)
		{	rec_count =0;
			sendstring("!");
			if ( rec_buffer[0]=='+') rc_cmd = PLAY_NEXT;
			else if ( rec_buffer[0]=='-') rc_cmd = PLAY_PREV;
			else if ( rec_buffer[0]=='*') rc_cmd = PLAY_STOP;			
			else if ( rec_buffer[0]=='n') rc_cmd = SHOW_NEXT;			
			else if ( rec_buffer[0]=='p') rc_cmd = SHOW_PREV;						
		}
		switch(rc_cmd)
		{
			case PLAY_NEXT:
					if((song_num_tmp != song_num))
					{
						//use as ENTER
						timer1=0;
						song_num = song_num_tmp;
//						song_num_tmp = 0;
						goto seek_song;
					}
song_up:
					get_next_song();
					if(song_num == 0)
					{
						//hmm, nix folgt.. versuchen wir es von vorne
						song_num++;
						get_next_song();
						if(song_num == 0) break;	//not found ???
					}
					goto seek_song;
					break;

			case PLAY_PREV:
//song_down:
					get_prev_song();
					if(song_num == 0)
					{
						//hmm, nix vorher ? nochmal !
						song_num = DirCount+1;
						get_prev_song();
						if(song_num == 0) break;	//not found ?
					}

seek_song:
					ST_REG=0;
					//check if dir !!!
					ptr_entry = get_song(song_num);
					if(is_entry_dir(ptr_entry) == 1)
					{
						c_ptr = (unsigned char *)&ptr_entry[2];
						if(  c_ptr[1] != 0x2e)
						{
							//add dirname
							strcat(dir_name, c_ptr);
						}
						else
						{
							//remove string
							stmp = strlen(dir_name);
							stmp--;
							while( dir_name[stmp] != DIR_MARK)
							{
								if(dir_name[stmp] == 0x00) break;
								stmp--;
							}
							dir_name[stmp] = 0x00;
						}
						dir(ptr_entry[0], DEVICE);
						timer1 = 0xFFFF;

						song_num_tmp = 1;
						song_num = 0;
						if((DirCount > 1) && (dir_buffer[8] != 0x2e))
						{
							song_num_tmp++;		//show first entry NOT ..
						}
//						song_num_tmp = song_num;
						show_song_tmp(song_num_tmp);
						break;
					}

					//start play
					play_file(show_play_title(), DEVICE);
					//mpginfo = mpg_info();
					break;

			case PLAY_STOP:
						ST_REG = 0;			//play stop
//						timer1=0;
						break;

			case SHOW_NEXT:
					if( song_num_tmp +1 > DirCount)
					{
						song_num_tmp = 1;
					}
					else
					{
						song_num_tmp++;
					}
					goto show_tmp_song1;
					break;

			case SHOW_PREV:
					if( (song_num_tmp == 0) ||( song_num_tmp == 1))
					{
						song_num_tmp = DirCount;
					}else
					{
						song_num_tmp--;
					}
show_tmp_song1:
					timer1 = 0xFFFF;
					show_song_tmp(song_num_tmp);
					break;


		}//end switch(rc_cmd)
	}//end if(new_ir)

	//next title ?
	if( (ST_REG & ((1<<PLAY_ENABLE) | (1<<ST_PLAYING)) ) == (1<<PLAY_ENABLE) )
	{
		//song ente -> play next song
		goto song_up;
	}

#endif


 }//end of for(;;)

//lcd_clr_display();
//lcd_print("!");
//while(1);
}//end of main

unsigned long * show_play_title(void)
{
	unsigned long *long_ptr;

	lcd_clr_display();

	song_num_tmp = song_num;	//for listing
	long_ptr = get_song(song_num);

	print_song_name( (unsigned char *)&long_ptr[2]);

	return long_ptr;
}


//setzt song_num auf nummer des naechsten MPEG eintrags
// song_num= 0 wenn kein naehster eintrag
void get_next_song(void)
{
	unsigned long *long_ptr;

	song_num++;
loopi:
	if(song_num > DirCount)
	{
		song_num=0;
		return;
	}

	long_ptr = get_song(song_num);
	if(*(unsigned char *)&long_ptr[2] == DIR_MARK)
	{
		song_num++;
		goto loopi;
	}
}

//setzt song_num auf nummer des vorherigen MPEG eintrags
// song_num= 0 wenn kein eintrag
void get_prev_song(void)
{
	unsigned long *long_ptr;

	//ed gibt keinen eintrag der vor dem ersten oder nullten kommt
	if( song_num < 2)
	{
		song_num = 0;
		return;
	}

	//song_num > 1
	song_num--;
loopi:
	long_ptr = get_song(song_num);
	if(*(unsigned char *)&long_ptr[2] == DIR_MARK)
	{
		song_num--;
		if(song_num != 0)	goto loopi;
	}
}


//returns 1 if dir; 0 else
unsigned char is_entry_dir(unsigned long *entry)
{
	if(*(unsigned char *)&entry[2] == DIR_MARK) return 1;
	return 0;
}

void print_song_name( unsigned char * name)
{
	lcd_print( name);	//show line 1
	lcd_set_addr(LCD_ADR_L1);
	lcd_print(dir_name);
}

//return ptr auf entry num song_num
unsigned long * get_song( unsigned short song_num)
{
	unsigned short tmp;
	unsigned char *tmp1;

	tmp = 1;
	tmp1= dir_buffer;
	while(tmp != song_num)
	{
		tmp1 = tmp1 + 8;
		while( *tmp1 != 0) tmp1++;
		tmp1++;
		tmp++;
	}
	return (unsigned long *)tmp1;
}

void show_song_tmp(unsigned short song_num_tmp)
{
	unsigned long *long_ptr;

	lcd_clr_display();
	lcd_print(byte_to_dec(song_num_tmp,0));
	lcd_print(" ");
	long_ptr = get_song( song_num_tmp);
	print_song_name( (unsigned char *)&long_ptr[2]);
}



