#include <io.h>
#include <string-avr.h>
#include <progmem.h>
#include "my_defs.h"
#include "lcd.h"
#include "tools.h"
#include "i2c.h"

void wait_bsy( void )
{
#if LCD == 8
	GO_WS;
	while (*LCD_CMD & 0x80);
	i2c_delay(10);
#endif
}

#if LCD == 4
	void lcd (unsigned char x)
	{
		*LCD_DATA = x;			//set data
		i2c_delay(50);			//waiting
		*LCD_DATA = x | 0x10;	//set lcd enable high
		i2c_delay(10);			//
		*LCD_DATA = x;			//clear lcd enable, but keep data
	}
#endif

void inline lcd_init( void)
{
	delay_ms(20);
#if LCD == 8
	GO_WS;
	*LCD_CMD = 0x38;		//set lines etc.
	wait_bsy();
	*LCD_CMD = 0x0C;		//display on
	wait_bsy();
	*LCD_CMD = 0x06;		//increment, no shift
	wait_bsy();
	*LCD_CMD = 0x01;		//cmd clear display
	END_WS;

#elif LCD == 4

	lcd_command(0x33);		//set interface to 4 bit
	delay_ms(5);
	lcd_command(0x33);		//set interface to 4 bit
	lcd_command(0x33);		//set interface to 4 bit
	lcd_command(0x02);
	lcd_command(0x28);		//set lines etc.
	lcd_command(0x0c);		//display on
	lcd_command(0x06);		//increment, no shift
	lcd_command(0x01);		//cmd clear display
#endif
}

void lcd_command( unsigned char command)
{
#if LCD == 8
	wait_bsy();
	*LCD_CMD = command;	
	END_WS;
#elif LCD == 4
	lcd((command >>4));
	lcd(command & 0x0f);
	delay_ms(2);
#endif
}

/*
void lcd_print( unsigned char *string)
{
	lcd_nprint( string, strlen(string));
}
*/
void lcd_nprint( unsigned char *string,unsigned char count)
{
#if LCD
unsigned char dummy, tmp;

	for( dummy = 0; dummy < count; dummy++)
	{
		tmp = *(string++);
		switch(tmp)
		{
			case 154:		//
			case 252:		//
			case 220:		//
				tmp = 0xF5;
			break;
			case 142:		//
			case 228:		//
			case 196:		//
				tmp = 0xE1;				   
			break;
			case 153:		//
			case 246:		//
			case 214:		//
				tmp = 0xEF;
			break;
			case 225:
			case 223:
				tmp = 0xE2;
			break;
			case 0x5c:
				tmp = 0xA5;
			break;
			case 0:	return;
		}
#if LCD == 8
		wait_bsy();
		*LCD_DATA = tmp;		
		END_WS;		
#elif LCD == 4
		lcd((tmp >> 4)    | 0x40);
		lcd((tmp & 0x0f)  | 0x40);
#endif		
	}
#endif
}

#if DEBUG == TRUE
void lcd_nprint_( unsigned char *string,unsigned char count)
{
	unsigned char dummy, tmp;

	for( dummy = 0; dummy < count; dummy++)
	{
		tmp = PRG_RDB(string++);
		switch(tmp)
		{
			case 154:		//
			case 252:		//
			case 220:		//
				tmp = 0xF5;
			break;
			case 142:		//
			case 228:		//
			case 196:		//
				tmp = 0xE1;
			break;
			case 153:		//
			case 246:		//
			case 214:		//
				tmp = 0xEF;
			break;
			case 225:
			case 223:
				tmp = 0xE2;
			break;
			case 0x5c:
				tmp = 0xA5;
			break;
			case 0:	return;
		}
#if LCD == 8
		wait_bsy();
		*LCD_DATA = tmp;
		END_WS;
#elif LCD == 4
		lcd((tmp >> 4)  | 0x60);
		lcd((tmp & 0x0f)  | 0x60);
#endif		

	}
}
#endif

/*
void lcd_set_addr( unsigned char addr)
{
	wait_bsy();
	*LCD_CMD = (addr | 0x80);
	END_WS;
}
*/