
#include "my_defs.h"
#include "memory.h"
#include "serial.h"
#include "tools.h"
#include "i2c.h"
#include "mas.h"

unsigned char _local_buffer[6];

unsigned long* mas_buffer = (unsigned long*)MASBUFFER_ADR;
unsigned long convert_mem(void);

void send_cmd(unsigned char cmd )
{
	i2c_start();
	i2c_write(MAS_DEV_WRITE);
	i2c_write(cmd);
}

void write_reg( unsigned char reg, unsigned long data)
{
//	0000 00d4 d3d2 d1d0

	_local_buffer[0] = 0x90 | (reg >> 4); 		//opcode + r1
	_local_buffer[1] = (reg << 4) | (unsigned char)(data & 0x0F);
	_local_buffer[2] = data >> 12;	//d4 + d3
	_local_buffer[3] = data >> 4;	//d2 + d1

	send_cmd( MAS_DATA_WRITE);
	i2c_send(4);
	i2c_stop();
}

unsigned long read_reg ( unsigned char reg)
{
	send_cmd( MAS_DATA_WRITE);

	_local_buffer[0] = 0xD0 | (reg >> 4);
	_local_buffer[1] = reg << 4;

	i2c_send(2);

	i2c_restart();				//		read data
	send_cmd( MAS_DATA_READ);

	i2c_restart();
	i2c_write(MAS_DEV_READ );

	i2c_receive (4);
	return convert_mem();
}


unsigned short default_read( void)
{
	send_cmd( MAS_DATA_READ);
	i2c_restart();
	i2c_write(MAS_DEV_READ );
	i2c_receive (2);
	return (unsigned short)(_local_buffer[0] << 8) | _local_buffer[1];
}

unsigned long convert_mem()
{
	return ((unsigned long) (_local_buffer[3] & 0x0F) << 16) | ((unsigned int)(_local_buffer[0] << 8)) | _local_buffer[1];
}

void mas_run( unsigned short addr)
{
	send_cmd( MAS_DATA_WRITE);
	_local_buffer[0] = addr >> 8;
	_local_buffer[1] = addr;
	i2c_send(2);
	i2c_stop();
}

unsigned long mas_read_mem( unsigned char mem, unsigned short addr)
{
	_local_buffer[0] = mem & 0xF0;
	_local_buffer[1] = 0x00;
	_local_buffer[2] = 0;
	_local_buffer[3] = 1;
	_local_buffer[4] = addr >> 8;
	_local_buffer[5] = addr;

	send_cmd( MAS_DATA_WRITE);
	i2c_send(6);
	i2c_stop();
	i2c_delay(HP);
	send_cmd( MAS_DATA_READ);
	i2c_restart();
	i2c_write( MAS_DEV_READ);

	i2c_receive (4);
	return convert_mem();
}

void mas_write_mem( unsigned char mem, unsigned short addr, unsigned long data)
{
											// init cmd  (select memory)
	_local_buffer[0] = mem & 0xB0; 			// convert cmd to write (bad...)
	_local_buffer[1] = 0x00;
	_local_buffer[2] = 0;					//word count
	_local_buffer[3] = 1;

	_local_buffer[4] = addr >> 8;			//set addr
	_local_buffer[5] = addr;

	send_cmd( MAS_DATA_WRITE);

	i2c_send(6);							// send header

	_local_buffer[0] = data >>8;			//d3,d2		// send data
	_local_buffer[1] = data;				//d1,d0
	_local_buffer[2] = 0x00;				//0,0
	_local_buffer[3] = (data >>16) & 0x0F;	//0,d4

	i2c_send(4);
	i2c_stop();
}

void inline mas_init ( void)
{
#if MINIMEGAHMPEG == FALSE	// Hmpeg
	outp(0<<PB0 | 0<<PB1 | 0<<PB2 | 0<<PB3 | 1<<PB4 | 1<<PB5 | 0<<PB6 | 1<<PB7, DDRB);			
	outp(1<<PB0 | 1<<PB1 | 1<<PB2 | 1<<PB3 | 1<<PB4 | 0<<PB5 | 0<<PB6 | 0<<PB7, PORTB);	//enable internal pull up resistor
	// Pullup auf dem IR kann nichts schaden [Oli]

	outp(1<<CPHA | 1<<MSTR | 1<<SPE, SPCR);								//set spi master, 2MHz clockrate
	
	outp(0<<PD0 | 1<<PD1 | 0<<PD2 | 0<<PD3 | 1<<PD4 | 1<<PD5 | 0<<PD6 | 0<<PD7, DDRD);
	// RXD = Eingang (eigentlich egal, wird eh ignoriert wenn UART enabled ist)
	// aber IDE_RESET muss Ausgang sein! [Oli]
	outp(1<<PD0 | 1<<PD1 | 0<<PD2 | 1<<PD3 | 1<<PD4 | 1<<PD5 | 0<<PD6 | 0<<PD7, PORTD);

#else	// MiniMegaHmpeg

	outp(1<<PB0 | 1<<PB1 | 1<<PB2 | 0<<PB3 | 0<<PB4 | 0<<PB5 | 0<<PB6 | 0<<PB7, DDRB);			
	outp(1<<PB0 | 0<<PB1 | 0<<PB2 | 0<<PB3 | 1<<PB4 | 1<<PB5 | 1<<PB6 | 1<<PB7, PORTB);	//enable internal pull up resistor

	outp(1<<CPHA | 1<<MSTR | 1<<SPE, SPCR);								//set spi master, 1.5MHz clockrate
	
	outp(0<<PD0 | 0<<PD1 | 0<<PD2 | 0<<PD3 | 0<<PD4 | 1<<PD5 | 1<<PD6 | 0<<PD7, DDRD);
	outp(1<<PD0 | 1<<PD1 | 1<<PD2 | 1<<PD3 | 1<<PD4 | 1<<PD5 | 1<<PD6 | 1<<PD7, PORTD);

	outp(0<<PE0 | 1<<PE1 | 0<<PE2 | 0<<PE3 | 0<<PE4 | 0<<PE5 | 0<<PE6 | 0<<PE7, DDRE);
	outp(1<<PE0 | 1<<PE1 | 1<<PE2 | 1<<PE3 | 0<<PE4 | 1<<PE5 | 1<<PE6 | 1<<PE7, PORTE);
#endif


#if MASKE_G10 == TRUE
	// Init fr 14.318 MHz	
	#ifdef QUARZ
		#if QUARZ == 14318180
			mas_write_mem( MEM_D0, 0x36D, 0x5D9D0);
			mas_write_mem( MEM_D0, 0x36e, 0xCECF4);	//31316???
		#endif
	#endif
	// Init fr den MAS3507D G10
	write_reg (0x3B,0x00000020);
	//Settings bernehmen
	mas_run(0x475);

#elif MASKE_F10 == TRUE
	#ifdef QUARZ
		#if QUARZ == 14318180
			// Init fr 14.318 MHz
			mas_write_mem( MEM_D0, 0x32D, 0x5D9D0);
   			mas_write_mem( MEM_D0, 0x32e, 0xCECF4);
   		#endif
   	//Settings bernehmen
   	mas_run(0x475);
   	#endif	   
#endif

#if LOUDNESS == TRUE
	//hebt Hhen und Bsse um 15dB an.
	write_reg(0xe7,0xe9400);
	write_reg(0x6b,0x61800);
	write_reg(0x6f,0x5f800);

	//	Basisbreitenvergrerung
	mas_buffer[0] = 0x40000;
	mas_write_mem( MEM_D1, 0x7F9, mas_buffer[0]);
	mas_write_mem( MEM_D1, 0x7Fa, mas_buffer[0]);
#endif
}


#define MPEG_1	3
#define MPEG_2	2
#define MPEG_25	0
#if 0
unsigned short mpg_info( void)
{
	unsigned short ret_val=0;
	unsigned long schrott;

	if(mas_read_mem(MEM_D0,0x301,mas_buffer) == 1) return 0;

	schrott = mas_buffer[0];
	ret_val = schrott >>11;
	ret_val = ret_val & 0x0F;	//xxxxMMLL M-MpegId L-LayerId

	if(mas_read_mem(MEM_D0, 0x302,mas_buffer) == 1) return 0;
	schrott = mas_buffer[0];
	ret_val = ret_val | ((schrott >> 6) &0x3f0);
	//xxxxxxSSSSFFMMLL	S-SamplingRate F-SamplingFrequeny

	return ret_val ;
}

void mpg_info( void)
{
	unsigned long *mp_status;
	unsigned char dummy,mp,layer;

	static unsigned short mp1_l2[] = {0,32,48,56,64,80,96,112,128,160,192,224,256,320,384};
	static unsigned short mp1_l3[] = {0,32,40,48,56,64,80,96,112,128,160,192,224,256,320};
	static unsigned char mp2_l23[]= {0,8,16,24,32,40,48,56,64,80,96,112,128,144,160};

	static unsigned char *mp1_f[] = {"44.1","48","32",""};
	static unsigned char *mp2_f[] = {"22.05","24","16",""};
	static unsigned char *mp25_f[]= {"11.025","12","8",""};


	mas_read_mem(MEM_D0, 0x301,mas_buffer);
	mp_status = (unsigned long *)mas_buffer;
	mp = (*mp_status >> 13) & 0x03;


	strcpy(w_buffer,"MPG ");

	switch(mp)
	{
		case MPEG_25:
			strcat(w_buffer,"2.5 ");
			break;
		case MPEG_2:
			strcat(w_buffer,"2 ");
			break;
		case MPEG_1:
			strcat(w_buffer,"1 ");
			break;
	}

	strcat(w_buffer,"Layer: ");
	layer = (*mp_status >> 11) & 0x03;
	switch(layer)
	{
		case 1:
			strcat(w_buffer,"3 ");
			break;
		case 2:
			strcat(w_buffer,"2 ");
			break;
		case 3:
			strcat(w_buffer,"1 ");
			break;
	}

	strcat(w_buffer,"\n\r");

	mas_read_mem(MEM_D0, 1, 0x302,mas_buffer+4);
	mp_status = (unsigned long *)mas_buffer+4;

	dummy = (*mp_status >> 12) & 0x0F;

	switch(mp)
	{
		//mpg 1
		case MPEG_1:
			if( layer == 2)
			{
				strcat(w_buffer, short_to_dec(mp1_l2[dummy]) );
			}
			else
			{
				strcat(w_buffer, short_to_dec(mp1_l3[dummy]) );
			}
			break;

		case MPEG_25:
		case MPEG_2:
				strcat(w_buffer, byte_to_dec(mp2_l23[dummy],0) );
			break;

	}
	strcat(w_buffer," kbs ");


	dummy = (*mp_status >> 10) & 0x03;

	switch(mp)
	{
		case MPEG_1:

				strcat(w_buffer, mp1_f[dummy]);
				break;
		case MPEG_2:
				strcat(w_buffer, mp2_f[dummy]);
				break;
		case MPEG_25:
				strcat(w_buffer, mp25_f[dummy]);
				break;
	}

	strcat(w_buffer, " kHz\n\r");
	sendstring(w_buffer);

}

void mas_info( void)
{

	unsigned long dummy;

	dummy = read_reg(MAS_REG_DCCF);
	sendstring("DCCF : ");
	print_long(dummy);
	new_line();


	dummy = read_reg(MAS_REG_MUTE);
	sendstring("Mute: ");
	print_long(dummy);
	new_line();

	dummy = read_reg(MAS_REG_PIODATA);
	sendstring("PIODATA: ");
	print_long(dummy);
	new_line();

	dummy = read_reg(MAS_REG_StartUpConfig);
	sendstring("StUpCfg: ");
	print_long(dummy);
	new_line();

	dummy = read_reg(MAS_REG_KPRESCALE);
	sendstring("PreScale: ");
	print_long(dummy);
	new_line();

	dummy = read_reg(MAS_REG_KBASS);
	sendstring("Bass: ");
	print_long(dummy);
	new_line();

	dummy = read_reg(MAS_REG_KTREBLE);
	sendstring("Treble: ");
	print_long(dummy);
	new_line();

/*	mas_read_mem( MEM_D0, 8, 0x300, mas_buffer);
	dump_mem(dir_buffer);
	new_line();
	dump_mem(dir_buffer+16);
	new_line();
*/	
}


#endif
