/* my definitions */

#ifndef __MY_DEFS_H
#define __MY_DEFS_H


//variables
#define	TRUE	1
#define	FALSE	0		   

#define SERIAL 			TRUE
#define RC5				TRUE
#define DREHSCHALTER 	TRUE
#define DEBUG			FALSE
//#define QUARZ			14318180
#define QUARZ			14745600
#define MASKE_F10		TRUE
#define MASKE_G10		FALSE
#define LOUDNESS 		FALSE
#define SCROLL			FALSE
// keine LCD-Untersttzung mehr, wenn LCD auf FALSE gesetzt wird
// bei Verwendung der LCD-Zusatzplatine LCD auf 4 setzen, sonst auf 8
#define LCD		 		8

// folgende Zeile auf TRUE setzen, wenn ein AVR Mega103
// mit dem MiniMegaHmpeg-Adapter eingesetzt wird.
// Auerdem muss noch in 'makefile' 'MCU = -mega103' gesetzt werden!
#define MINIMEGAHMPEG	TRUE

// folgende Zeilen definieren die Quarzfrequenz des AVRs
// Normal ist 8 MHz beim Hmpeg und 6 MHz beim MiniMegaHmpeg
#if MINIMEGAHMPEG == FALSE
	#define F_CPU			8000000		// HMpeg
#else
	#define F_CPU			6000000		// MiniMegaHmpeg
#endif

// folgende Zeilen definieren die Baudrate der seriellen 
// Schnittstelle fr den HMpeg und den MiniHmpeg.
// Normal ist 38400 Baud beim Hmpeg und 9600 Baud beim MiniMegaHmpeg
#if MINIMEGAHMPEG == FALSE
	#define UART_BAUD_RATE	38400		// HMpeg
#else
	#define UART_BAUD_RATE	9600		// MiniMegaHmpeg
#endif


// Hmpeg benutzt timer0 fr diverse Zeitmessungen (IR, Delay),
// MiniMegaHmpeg benutzt timer2, damit timer0 als Echtzeituhr verwendbar bleibt
#if MINIMEGAHMPEG == FALSE				// HMpeg
	#define TIMER8BIT_START		timer0_start()
	#define TIMER8BIT_CNT		inp(TCNT0)
#else									// MiniMegaHmpeg
	#define TIMER8BIT_START		timer2_start()
	#define TIMER8BIT_CNT		inp(TCNT2)
#endif


#include <io.h>
#include <interrupt.h>
#if __GNUC__ == 2
	#include <signal.h>
#else
	#include <sig-avr.h>
#endif
#include <eeprom.h>
#include <string-avr.h>
#include <timer.h>

#define	DIR_MARK	0x5C
#define NL		"\n\r"

//#define	DEVICE	CB_DH_DEV0

//ST_REG bit definitions
#define	ST_PLAYING			1
#define	STOP_AFTER_PLAYING	2
#define	PLAY_ENABLE			3
#define PAUSE				4
#define STOP				0

#define SECTORSIZE 			512

//nur zweizeilige Displays
#define LCD_ADR_L0		(0x00 | 0x80)
#define LCD_ADR_L1		(0x40 | 0x80)
#define LCD_SPALTEN 	40

//serielle Kommandos
#define PLAY_NEXT				'+'
#define PLAY_PREV				'-'
#define SHOW_NEXT				'?'
#define SET_PREV				'p'
#define SET_NEXT				'n'
#define PLAY					'x'
#define PLAY_STOP				' '
#define NEXT_DIR				'w'
#define PREV_DIR				'z'
#define RESUME					'r'
#define SAVE					's'
#define PLAY_ENTRY				'e'
#define DIR_CLUSTER				'd'
#define INFO					'i'
#define READ_REG_MAS 			'4'
#define READ_MEM_MAS			'5'
#define RUN_ADR_MAS				'6'
#define WRITE_MEM_MAS			'7'
#define WRITE_REG_MAS			'8'
#define READ_DEFAULT			'9'
#define GO						0x0d
#define SEND_ID3TAG				'I'
#define SEND_PLAYLIST			'S'
#define NOP						'N'
#define READY					0x0d
#define SEND_STATUS				'm'
#define WRITE_STATUS			'M'


//Hardwareeinstellungen Eingnge

#define PLAYBUTTON_PRESSED		bit_is_clear(PIND, PIND3)
#define PLAYBUTTON_NOT_PRESSED	bit_is_set(PIND, PIND3)

#if MINIMEGAHMPEG == FALSE	// Hmpeg

	#define TRACK_A 				bit_is_set(PINB, PINB2)
	#define TRACK_B 				bit_is_set(PINB, PINB3)
	#define REMOTE_CLEAR			bit_is_clear(PINB, PINB0)
	#define REMOTE_SET				bit_is_set(PINB, PINB0)
	
	#define DEMAND_SET				bit_is_set(PIND, PIND2)
	#define DEMAND_CLEAR			bit_is_clear(PIND, PIND2)
	#define RESET_SET				bit_is_set(PINB, PINB4)
	#define SID_SET					bit_is_set(PINB, PINB5)
	#define SIC_SET					bit_is_set(PINB, PINB7)
	
	#define I2CC_SET				bit_is_set(PIND, PIND5)
	#define I2CD_SET				bit_is_set(PINB, PINB1)

#else	// MiniMegaHmpeg

	#define TRACK_A 				bit_is_set(PIND, PIND2)
	#define TRACK_B 				bit_is_set(PIND, PIND4)
	#define REMOTE_CLEAR			bit_is_clear(PINE, PINE5)
	#define REMOTE_SET				bit_is_set(PINE, PINE5)

	#define DEMAND_SET				bit_is_set(PINE, PINE4)
	#define DEMAND_CLEAR			bit_is_clear(PINE, PIND4)
	#define RESET_SET				bit_is_set(PINB, PINB0)
	#define SID_SET					bit_is_set(PINB, PINB2)
	#define SIC_SET					bit_is_set(PINB, PINB1)

	#define I2CC_SET				bit_is_set(PIND, PIND6)
	#define I2CD_SET				bit_is_set(PIND, PIND7)
#endif

//Hardwareeinstellungen Ausgnge

// Funktionen die beim Compiler fehlen
#if MINIMEGAHMPEG == TRUE	// MiniMegaHmpeg

static inline void timer2_source (unsigned int src)
{
  outb (src, TCCR2);
}

static inline void timer2_start (void)
{
  outb (0x1, TCNT2);
}
#endif

// assembler macros
#define GO_WS	 sbi(MCUCR,SRW);
//#define GO_WS outp(_MCU_REG_WS, MCUCR)

#define END_WS	 cbi( MCUCR, SRW);
//#define END_WS outp(_MCU_REG_NOWS, MCUCR)

#include <inttypes.h>
#define nop()  asm volatile ("nop" ::)

#define cbr(port, bit)				\
	asm volatile (					\
		"cbr %1, %2"				\
		: "=r" ((uint8_t)(port))		\
		: "r" ((uint8_t)(port)),		\
		  "I" ((uint8_t)(bit))			\
	)


#define sbr(port, bit)				\
	asm volatile (					\
		"sbr %1, %2"				\
		: "=r" ((uint8_t)(port))		\
		: "r" ((uint8_t)(port)),		\
		  "I" ((uint8_t)(bit))			\
	)


#define rol(reg)				\
	asm volatile (					\
		"rol %0"				\
		: "=r"((uint8_t)(reg))			\
		: "r" ((uint8_t)(reg))	\
	)
#define lsr(reg)				\
	asm volatile (					\
		"lsr %0"				\
		: "=r"((uint8_t)(reg))			\
		: "r" ((uint8_t)(reg))	\
	)

#define sec() asm volatile("sec" ::)
#define clc() asm volatile("clc" ::)

#define carry_set() ({\
	uint8_t t;\
	asm volatile (\
		"clr %0" "\n\t"\
		"brcc L_%=" "\n\t"\
		"inc %0" "\n"	\
		"L_%=: " "\n\t" \
		: "=r" (t)\
		:\
		);\
		t;\
	})



#endif
